@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -106,6 +106,12 @@ public class ErrorCode {
      */
     public static final int NUMERIC_VALUE_OUT_OF_RANGE_1 = 22003;
 
+    /**
+     * The error with code <code>22004</code> is thrown when a value is out of
+     * range when converting to another column's data type.
+     */
+    public static final int NUMERIC_VALUE_OUT_OF_RANGE_2 = 22004;
+
     /**
      * The error with code <code>22007</code> is thrown when
      * a text can not be converted to a date, time, or timestamp constant.
@@ -165,34 +171,13 @@ public class ErrorCode {
      * but the value is not one of the values enumerated by the
      * type.
      *
-     * This error is best thrown in a context when the column name
-     * and it's enumerated values are known.
-     *
      * Example:
      * <pre>
      * CREATE TABLE TEST(CASE ENUM('sensitive','insensitive'));
      * INSERT INTO TEST VALUES('snake');
      * </pre>
      */
-    public static final int ENUM_VALUE_NOT_PERMITTED_1 = 22030;
-
-    /**
-     * The error with code <code>22031</code> is typically thrown
-     * when a math operation is attempted on an ENUM-typed cell,
-     * but the value resulting from the operation is not one of
-     * values enumerated by the type.
-     *
-     * This error is best thrown in a context when the column name
-     * is not known, but the enumerated values of the type are known.
-     *
-     * Example:
-     * <pre>
-     * CREATE TABLE TEST(CASE ENUM('sensitive','insensitive'));
-     * INSERT INTO TEST VALUES('sensitive');
-     * UPDATE TEST SET CASE = CASE + 100;
-     * </pre>
-     */
-    public static final int ENUM_VALUE_NOT_PERMITTED_2 = 22031;
+    public static final int ENUM_VALUE_NOT_PERMITTED = 22030;
 
     /**
      * The error with code <code>22032</code> is thrown when an
