@@ -1,11 +1,13 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.engine;
 
+import java.util.ArrayDeque;
 import java.util.ArrayList;
+import java.util.Deque;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -42,6 +44,7 @@ import org.h2.table.SubQueryInfo;
 import org.h2.table.Table;
 import org.h2.table.TableFilter;
 import org.h2.table.TableType;
+import org.h2.util.ColumnNamerConfiguration;
 import org.h2.util.New;
 import org.h2.util.SmallLRUCache;
 import org.h2.value.Value;
@@ -82,6 +85,7 @@ public class Session extends SessionWithState {
     private Value lastIdentity = ValueLong.get(0);
     private Value lastScopeIdentity = ValueLong.get(0);
     private Value lastTriggerIdentity;
+    private GeneratedKeys generatedKeys;
     private int firstUncommittedLog = Session.LOG_WRITTEN;
     private int firstUncommittedPos = Session.LOG_WRITTEN;
     private HashMap<String, Savepoint> savepoints;
@@ -120,12 +124,14 @@ public class Session extends SessionWithState {
     private long modificationMetaID = -1;
     private SubQueryInfo subQueryInfo;
     private int parsingView;
+    private final Deque<String> viewNameStack = new ArrayDeque<>();
     private int preparingQueryExpression;
     private volatile SmallLRUCache<Object, ViewIndex> viewIndexCache;
     private HashMap<Object, ViewIndex> subQueryIndexCache;
     private boolean joinBatchEnabled;
     private boolean forceJoinOrder;
     private boolean lazyQueryExecution;
+    private ColumnNamerConfiguration columnNamerConfiguration;
     /**
      * Tables marked for ANALYZE after the current transaction is committed.
      * Prevents us calling ANALYZE repeatedly in large transactions.
@@ -162,6 +168,7 @@ public class Session extends SessionWithState {
         this.lockTimeout = setting == null ?
                 Constants.INITIAL_LOCK_TIMEOUT : setting.getIntValue();
         this.currentSchemaName = Constants.SCHEMA_MAIN;
+        this.columnNamerConfiguration = ColumnNamerConfiguration.getDefault();
     }
 
     public void setLazyQueryExecution(boolean lazyQueryExecution) {
@@ -223,13 +230,35 @@ public class Session extends SessionWithState {
         return subQueryInfo;
     }
 
-    public void setParsingView(boolean parsingView) {
+    /**
+     * Stores name of currently parsed view in a stack so it can be determined
+     * during {@code prepare()}.
+     *
+     * @param parsingView
+     *            {@code true} to store one more name, {@code false} to remove it
+     *            from stack
+     * @param viewName
+     *            name of the view
+     */
+    public void setParsingCreateView(boolean parsingView, String viewName) {
         // It can be recursive, thus implemented as counter.
         this.parsingView += parsingView ? 1 : -1;
         assert this.parsingView >= 0;
+        if (parsingView) {
+            viewNameStack.push(viewName);
+        } else {
+            assert viewName.equals(viewNameStack.peek());
+            viewNameStack.pop();
+        }
+    }
+    public String getParsingCreateViewName() {
+        if (viewNameStack.isEmpty()) {
+            return null;
+        }
+        return viewNameStack.peek();
     }
 
-    public boolean isParsingView() {
+    public boolean isParsingCreateView() {
         assert parsingView >= 0;
         return parsingView != 0;
     }
@@ -261,7 +290,7 @@ public class Session extends SessionWithState {
 
     @Override
     public ArrayList<String> getClusterServers() {
-        return new ArrayList<String>();
+        return new ArrayList<>();
     }
 
     public boolean setCommitOrRollbackDisabled(boolean x) {
@@ -323,9 +352,7 @@ public class Session extends SessionWithState {
         if (variables == null) {
             return new String[0];
         }
-        String[] list = new String[variables.size()];
-        variables.keySet().toArray(list);
-        return list;
+        return variables.keySet().toArray(new String[variables.size()]);
     }
 
     /**
@@ -346,7 +373,7 @@ public class Session extends SessionWithState {
         if (localTempTables == null) {
             return New.arrayList();
         }
-        return New.arrayList(localTempTables.values());
+        return new ArrayList<>(localTempTables.values());
     }
 
     /**
@@ -361,7 +388,7 @@ public class Session extends SessionWithState {
         }
         if (localTempTables.get(table.getName()) != null) {
             throw DbException.get(ErrorCode.TABLE_OR_VIEW_ALREADY_EXISTS_1,
-                    table.getSQL());
+                    table.getSQL()+" AS "+table.getName());
         }
         modificationId++;
         localTempTables.put(table.getName(), table);
@@ -373,6 +400,9 @@ public class Session extends SessionWithState {
      * @param table the table
      */
     public void removeLocalTempTable(Table table) {
+        // Exception thrown in org.h2.engine.Database.removeMeta if line below
+        // is missing with TestGeneralCommonTableQueries
+        database.lockMeta(this);
         modificationId++;
         localTempTables.remove(table.getName());
         synchronized (database) {
@@ -396,7 +426,7 @@ public class Session extends SessionWithState {
 
     public HashMap<String, Index> getLocalTempTableIndexes() {
         if (localTempTableIndexes == null) {
-            return New.hashMap();
+            return new HashMap<>();
         }
         return localTempTableIndexes;
     }
@@ -454,7 +484,7 @@ public class Session extends SessionWithState {
      */
     public HashMap<String, Constraint> getLocalTempTableConstraints() {
         if (localTempTableConstraints == null) {
-            return New.hashMap();
+            return new HashMap<>();
         }
         return localTempTableConstraints;
     }
@@ -527,7 +557,7 @@ public class Session extends SessionWithState {
      * @return the prepared statement
      */
     public Prepared prepare(String sql) {
-        return prepare(sql, false);
+        return prepare(sql, false, false);
     }
 
     /**
@@ -535,11 +565,14 @@ public class Session extends SessionWithState {
      *
      * @param sql the SQL statement
      * @param rightsChecked true if the rights have already been checked
+     * @param literalsChecked true if the sql string has already been checked
+     *            for literals (only used if ALLOW_LITERALS NONE is set).
      * @return the prepared statement
      */
-    public Prepared prepare(String sql, boolean rightsChecked) {
+    public Prepared prepare(String sql, boolean rightsChecked, boolean literalsChecked) {
         Parser parser = new Parser(this);
         parser.setRightsChecked(rightsChecked);
+        parser.setLiteralsChecked(literalsChecked);
         return parser.prepare(sql);
     }
 
@@ -618,9 +651,8 @@ public class Session extends SessionWithState {
             // increment the data mod count, so that other sessions
             // see the changes
             // TODO should not rely on locking
-            if (locks.size() > 0) {
-                for (int i = 0, size = locks.size(); i < size; i++) {
-                    Table t = locks.get(i);
+            if (!locks.isEmpty()) {
+                for (Table t : locks) {
                     if (t instanceof MVTable) {
                         ((MVTable) t).commit();
                     }
@@ -646,8 +678,7 @@ public class Session extends SessionWithState {
                         rows.add(entry.getRow());
                         undoLog.removeLast(false);
                     }
-                    for (int i = 0, size = rows.size(); i < size; i++) {
-                        Row r = rows.get(i);
+                    for (Row r : rows) {
                         r.commit();
                     }
                 }
@@ -663,15 +694,18 @@ public class Session extends SessionWithState {
                 autoCommitAtTransactionEnd = false;
             }
         }
-        endTransaction();
 
         int rows = getDatabase().getSettings().analyzeSample / 10;
         if (tablesToAnalyze != null) {
             for (Table table : tablesToAnalyze) {
                 Analyze.analyzeTable(this, table, rows, false);
             }
+            // analyze can lock the meta
+            database.unlockMeta(this);
         }
         tablesToAnalyze = null;
+
+        endTransaction();
     }
 
     private void removeTemporaryLobs(boolean onTimeout) {
@@ -689,10 +723,10 @@ public class Session extends SessionWithState {
             }
             temporaryLobs.clear();
         }
-        if (temporaryResultLobs != null && temporaryResultLobs.size() > 0) {
+        if (temporaryResultLobs != null && !temporaryResultLobs.isEmpty()) {
             long keepYoungerThan = System.nanoTime() -
                     TimeUnit.MILLISECONDS.toNanos(database.getSettings().lobTimeout);
-            while (temporaryResultLobs.size() > 0) {
+            while (!temporaryResultLobs.isEmpty()) {
                 TimeoutValue tv = temporaryResultLobs.getFirst();
                 if (onTimeout && tv.created >= keepYoungerThan) {
                     break;
@@ -706,7 +740,7 @@ public class Session extends SessionWithState {
     }
 
     private void checkCommitRollback() {
-        if (commitOrRollbackDisabled && locks.size() > 0) {
+        if (commitOrRollbackDisabled && !locks.isEmpty()) {
             throw DbException.get(ErrorCode.COMMIT_ROLLBACK_NOT_ALLOWED);
         }
     }
@@ -732,6 +766,7 @@ public class Session extends SessionWithState {
     public void rollback() {
         checkCommitRollback();
         currentTransactionName = null;
+        transactionStart = 0;
         boolean needCommit = false;
         if (undoLog.size() > 0) {
             rollbackTo(null, false);
@@ -745,7 +780,7 @@ public class Session extends SessionWithState {
             transaction.commit();
             transaction = null;
         }
-        if (locks.size() > 0 || needCommit) {
+        if (!locks.isEmpty() || needCommit) {
             database.commit(this);
         }
         cleanTempTables(false);
@@ -796,8 +831,7 @@ public class Session extends SessionWithState {
             }
         }
         if (savepoints != null) {
-            String[] names = new String[savepoints.size()];
-            savepoints.keySet().toArray(names);
+            String[] names = savepoints.keySet().toArray(new String[savepoints.size()]);
             for (String name : names) {
                 Savepoint sp = savepoints.get(name);
                 int savepointIndex = sp.logIndex;
@@ -848,6 +882,10 @@ public class Session extends SessionWithState {
                 removeTemporaryLobs(false);
                 cleanTempTables(true);
                 undoLog.clear();
+                // Table#removeChildrenAndResources can take the meta lock,
+                // and we need to unlock before we call removeSession(), which might
+                // want to take the meta lock using the system session.
+                database.unlockMeta(this);
                 database.removeSession(this);
             } finally {
                 closed = true;
@@ -890,7 +928,7 @@ public class Session extends SessionWithState {
                 if (lockMode != Constants.LOCK_MODE_OFF &&
                         !database.isMultiVersion()) {
                     TableType tableType = log.getTable().getTableType();
-                    if (locks.indexOf(log.getTable()) < 0
+                    if (!locks.contains(log.getTable())
                             && TableType.TABLE_LINK != tableType
                             && TableType.EXTERNAL_TABLE_ENGINE != tableType) {
                         DbException.throwInternalError("" + tableType);
@@ -902,8 +940,7 @@ public class Session extends SessionWithState {
             if (database.isMultiVersion()) {
                 // see also UndoLogRecord.commit
                 ArrayList<Index> indexes = table.getIndexes();
-                for (int i = 0, size = indexes.size(); i < size; i++) {
-                    Index index = indexes.get(i);
+                for (Index index : indexes) {
                     index.commit(operation, row);
                 }
                 row.commit();
@@ -948,14 +985,14 @@ public class Session extends SessionWithState {
                 DbException.throwInternalError();
             }
         }
-        if (locks.size() > 0) {
+        if (!locks.isEmpty()) {
             // don't use the enhanced for loop to save memory
-            for (int i = 0, size = locks.size(); i < size; i++) {
-                Table t = locks.get(i);
+            for (Table t : locks) {
                 t.unlock(this);
             }
             locks.clear();
         }
+        database.unlockMetaDebug(this);
         savepoints = null;
         sessionStateChanged = true;
     }
@@ -970,6 +1007,9 @@ public class Session extends SessionWithState {
                         modificationId++;
                         table.setModified();
                         it.remove();
+                        // Exception thrown in org.h2.engine.Database.removeMeta
+                        // if line below is missing with TestDeadlock
+                        database.lockMeta(this);
                         table.removeChildrenAndResources(this);
                         if (closeSession) {
                             // need to commit, otherwise recovery might
@@ -980,11 +1020,6 @@ public class Session extends SessionWithState {
                         table.truncate(this);
                     }
                 }
-                // sometimes Table#removeChildrenAndResources
-                // will take the meta lock
-                if (closeSession) {
-                    database.unlockMeta(this);
-                }
             }
         }
     }
@@ -1034,6 +1069,13 @@ public class Session extends SessionWithState {
         return lastTriggerIdentity;
     }
 
+    public GeneratedKeys getGeneratedKeys() {
+        if (generatedKeys == null) {
+            generatedKeys = new GeneratedKeys();
+        }
+        return generatedKeys;
+    }
+
     /**
      * Called when a log entry for this session is added. The session keeps
      * track of the first entry in the transaction log that is not yet
@@ -1098,6 +1140,8 @@ public class Session extends SessionWithState {
      */
     public void rollbackToSavepoint(String name) {
         checkCommitRollback();
+        currentTransactionName = null;
+        transactionStart = 0;
         if (savepoints == null) {
             throw DbException.get(ErrorCode.SAVEPOINT_IS_INVALID_1, name);
         }
@@ -1114,9 +1158,6 @@ public class Session extends SessionWithState {
      * @param transactionName the name of the transaction
      */
     public void prepareCommit(String transactionName) {
-        if (transaction != null) {
-            database.prepareCommit(this, transactionName);
-        }
         if (containsUncommitted()) {
             // need to commit even if rollback is not possible (create/drop
             // table and so on)
@@ -1197,9 +1238,20 @@ public class Session extends SessionWithState {
      * executing the statement.
      *
      * @param command the command
+     * @param generatedKeysRequest
+     *            {@code false} if generated keys are not needed, {@code true} if
+     *            generated keys should be configured automatically, {@code int[]}
+     *            to specify column indices to return generated keys from, or
+     *            {@code String[]} to specify column names to return generated keys
+     *            from
      */
-    public void setCurrentCommand(Command command) {
+    public void setCurrentCommand(Command command, Object generatedKeysRequest) {
         this.currentCommand = command;
+        // Preserve generated keys in case of a new query due to possible nested
+        // queries in update
+        if (command != null && !command.isQuery()) {
+            getGeneratedKeys().clear(generatedKeysRequest);
+        }
         if (queryTimeout > 0 && command != null) {
             currentCommandStart = System.currentTimeMillis();
             long now = System.nanoTime();
@@ -1298,9 +1350,9 @@ public class Session extends SessionWithState {
             DbException.throwInternalError(v.toString());
         }
         if (removeLobMap == null) {
-            removeLobMap = New.hashMap();
-            removeLobMap.put(v.toString(), v);
+            removeLobMap = new HashMap<>();
         }
+        removeLobMap.put(v.toString(), v);
     }
 
     /**
@@ -1419,17 +1471,15 @@ public class Session extends SessionWithState {
     public Table[] getLocks() {
         // copy the data without synchronizing
         ArrayList<Table> copy = New.arrayList();
-        for (int i = 0; i < locks.size(); i++) {
+        for (Table lock : locks) {
             try {
-                copy.add(locks.get(i));
+                copy.add(lock);
             } catch (Exception e) {
                 // ignore
                 break;
             }
         }
-        Table[] list = new Table[copy.size()];
-        copy.toArray(list);
-        return list;
+        return copy.toArray(new Table[0]);
     }
 
     /**
@@ -1473,7 +1523,7 @@ public class Session extends SessionWithState {
             // not grow too large for a single query (we drop the whole cache in
             // the end of prepareLocal)
             if (subQueryIndexCache == null) {
-                subQueryIndexCache = New.hashMap();
+                subQueryIndexCache = new HashMap<>();
             }
             return subQueryIndexCache;
         }
@@ -1496,7 +1546,7 @@ public class Session extends SessionWithState {
             return;
         }
         if (temporaryResults == null) {
-            temporaryResults = New.hashSet();
+            temporaryResults = new HashSet<>();
         }
         if (temporaryResults.size() < 100) {
             // reference at most 100 result sets to avoid memory problems
@@ -1679,12 +1729,12 @@ public class Session extends SessionWithState {
         if (v.getTableId() == LobStorageFrontend.TABLE_RESULT ||
                 v.getTableId() == LobStorageFrontend.TABLE_TEMP) {
             if (temporaryResultLobs == null) {
-                temporaryResultLobs = new LinkedList<TimeoutValue>();
+                temporaryResultLobs = new LinkedList<>();
             }
             temporaryResultLobs.add(new TimeoutValue(v));
         } else {
             if (temporaryLobs == null) {
-                temporaryLobs = new ArrayList<Value>();
+                temporaryLobs = new ArrayList<>();
             }
             temporaryLobs.add(v);
         }
@@ -1702,7 +1752,7 @@ public class Session extends SessionWithState {
      */
     public void markTableForAnalyze(Table table) {
         if (tablesToAnalyze == null) {
-            tablesToAnalyze = New.hashSet();
+            tablesToAnalyze = new HashSet<>();
         }
         tablesToAnalyze.add(table);
     }
@@ -1744,4 +1794,18 @@ public class Session extends SessionWithState {
         }
 
     }
+
+    public ColumnNamerConfiguration getColumnNamerConfiguration() {
+        return columnNamerConfiguration;
+    }
+
+    public void setColumnNamerConfiguration(ColumnNamerConfiguration columnNamerConfiguration) {
+        this.columnNamerConfiguration = columnNamerConfiguration;
+    }
+
+    @Override
+    public boolean isSupportsGeneratedKeys() {
+        return true;
+    }
+
 }
