@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -145,10 +145,7 @@ public class User extends RightOwner {
                 return true;
             }
         }
-        if (isRightGrantedRecursive(table, rightMask)) {
-            return true;
-        }
-        return false;
+        return isRightGrantedRecursive(table, rightMask);
     }
 
     /**
