@@ -1,10 +1,11 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.fulltext;
 
+import java.io.File;
 import java.io.IOException;
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
@@ -13,18 +14,27 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
+import java.util.Map;
 import org.apache.lucene.analysis.Analyzer;
 import org.apache.lucene.analysis.standard.StandardAnalyzer;
 import org.apache.lucene.document.DateTools;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field;
 import org.apache.lucene.index.IndexReader;
+import org.apache.lucene.index.IndexWriter;
 import org.apache.lucene.index.IndexWriterConfig;
 import org.apache.lucene.index.Term;
 import org.apache.lucene.queryParser.QueryParser;
 import org.apache.lucene.search.IndexSearcher;
 import org.apache.lucene.search.Query;
+import org.apache.lucene.search.ScoreDoc;
+import org.apache.lucene.search.TopDocs;
+import org.apache.lucene.store.Directory;
+import org.apache.lucene.store.FSDirectory;
+import org.apache.lucene.store.RAMDirectory;
+import org.apache.lucene.util.Version;
 import org.h2.api.Trigger;
 import org.h2.command.Parser;
 import org.h2.engine.Session;
@@ -36,14 +46,6 @@ import org.h2.util.New;
 import org.h2.util.StatementBuilder;
 import org.h2.util.StringUtils;
 import org.h2.util.Utils;
-import java.io.File;
-import org.apache.lucene.search.ScoreDoc;
-import org.apache.lucene.search.TopDocs;
-import org.apache.lucene.store.FSDirectory;
-import org.apache.lucene.store.Directory;
-import org.apache.lucene.store.RAMDirectory;
-import org.apache.lucene.util.Version;
-import org.apache.lucene.index.IndexWriter;
 
 /**
  * This class implements the full text search based on Apache Lucene.
@@ -57,7 +59,7 @@ public class FullTextLucene extends FullText {
     protected static final boolean STORE_DOCUMENT_TEXT_IN_INDEX =
             Utils.getProperty("h2.storeDocumentTextInIndex", false);
 
-    private static final HashMap<String, IndexAccess> INDEX_ACCESS = New.hashMap();
+    private static final HashMap<String, IndexAccess> INDEX_ACCESS = new HashMap<>();
     private static final String TRIGGER_PREFIX = "FTL_";
     private static final String SCHEMA = "FTL";
     private static final String LUCENE_FIELD_DATA = "_DATA";
@@ -94,27 +96,23 @@ public class FullTextLucene extends FullText {
      * @param conn the connection
      */
     public static void init(Connection conn) throws SQLException {
-        Statement stat = conn.createStatement();
-        stat.execute("CREATE SCHEMA IF NOT EXISTS " + SCHEMA);
-        stat.execute("CREATE TABLE IF NOT EXISTS " + SCHEMA +
-                ".INDEXES(SCHEMA VARCHAR, TABLE VARCHAR, " +
-                "COLUMNS VARCHAR, PRIMARY KEY(SCHEMA, TABLE))");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FTL_CREATE_INDEX FOR \"" +
-                FullTextLucene.class.getName() + ".createIndex\"");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FTL_DROP_INDEX FOR \"" +
-                FullTextLucene.class.getName() + ".dropIndex\"");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FTL_SEARCH FOR \"" +
-                FullTextLucene.class.getName() + ".search\"");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FTL_SEARCH_DATA FOR \"" +
-                FullTextLucene.class.getName() + ".searchData\"");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FTL_REINDEX FOR \"" +
-                FullTextLucene.class.getName() + ".reindex\"");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FTL_DROP_ALL FOR \"" +
-                FullTextLucene.class.getName() + ".dropAll\"");
-        try {
-            getIndexAccess(conn);
-        } catch (SQLException e) {
-            throw convertException(e);
+        try (Statement stat = conn.createStatement()) {
+            stat.execute("CREATE SCHEMA IF NOT EXISTS " + SCHEMA);
+            stat.execute("CREATE TABLE IF NOT EXISTS " + SCHEMA +
+                    ".INDEXES(SCHEMA VARCHAR, TABLE VARCHAR, " +
+                    "COLUMNS VARCHAR, PRIMARY KEY(SCHEMA, TABLE))");
+            stat.execute("CREATE ALIAS IF NOT EXISTS FTL_CREATE_INDEX FOR \"" +
+                    FullTextLucene.class.getName() + ".createIndex\"");
+            stat.execute("CREATE ALIAS IF NOT EXISTS FTL_DROP_INDEX FOR \"" +
+                    FullTextLucene.class.getName() + ".dropIndex\"");
+            stat.execute("CREATE ALIAS IF NOT EXISTS FTL_SEARCH FOR \"" +
+                    FullTextLucene.class.getName() + ".search\"");
+            stat.execute("CREATE ALIAS IF NOT EXISTS FTL_SEARCH_DATA FOR \"" +
+                    FullTextLucene.class.getName() + ".searchData\"");
+            stat.execute("CREATE ALIAS IF NOT EXISTS FTL_REINDEX FOR \"" +
+                    FullTextLucene.class.getName() + ".reindex\"");
+            stat.execute("CREATE ALIAS IF NOT EXISTS FTL_DROP_ALL FOR \"" +
+                    FullTextLucene.class.getName() + ".dropAll\"");
         }
     }
 
@@ -157,11 +155,9 @@ public class FullTextLucene extends FullText {
         prep.setString(1, schema);
         prep.setString(2, table);
         int rowCount = prep.executeUpdate();
-        if (rowCount == 0) {
-            return;
+        if (rowCount != 0) {
+            reindex(conn);
         }
-
-        reindex(conn);
     }
 
     /**
@@ -191,7 +187,7 @@ public class FullTextLucene extends FullText {
      */
     public static void dropAll(Connection conn) throws SQLException {
         Statement stat = conn.createStatement();
-        stat.execute("DROP SCHEMA IF EXISTS " + SCHEMA);
+        stat.execute("DROP SCHEMA IF EXISTS " + SCHEMA + " CASCADE");
         removeAllTriggers(conn, TRIGGER_PREFIX);
         removeIndexFiles(conn);
     }
@@ -248,10 +244,7 @@ public class FullTextLucene extends FullText {
      * @return the converted SQL exception
      */
     protected static SQLException convertException(Exception e) {
-        SQLException e2 = new SQLException(
-                "Error while indexing document", "FULLTEXT");
-        e2.initCause(e);
-        return e2;
+        return new SQLException("Error while indexing document", "FULLTEXT", e);
     }
 
     /**
@@ -261,7 +254,7 @@ public class FullTextLucene extends FullText {
      * @param schema the schema name
      * @param table the table name
      */
-    protected static void createTrigger(Connection conn, String schema,
+    private static void createTrigger(Connection conn, String schema,
             String table) throws SQLException {
         createOrDropTrigger(conn, schema, table, true);
     }
@@ -309,11 +302,7 @@ public class FullTextLucene extends FullText {
                     conf.setOpenMode(IndexWriterConfig.OpenMode.CREATE_OR_APPEND);
                     IndexWriter writer = new IndexWriter(indexDir, conf);
                     //see http://wiki.apache.org/lucene-java/NearRealtimeSearch
-                    IndexReader reader = IndexReader.open(writer, true);
-                    access = new IndexAccess();
-                    access.writer = writer;
-                    access.reader = reader;
-                    access.searcher = new IndexSearcher(reader);
+                    access = new IndexAccess(writer);
                 } catch (IOException e) {
                     throw convertException(e);
                 }
@@ -353,12 +342,12 @@ public class FullTextLucene extends FullText {
      * @param schema the schema name
      * @param table the table name
      */
-    protected static void indexExistingRows(Connection conn, String schema,
+    private static void indexExistingRows(Connection conn, String schema,
             String table) throws SQLException {
         FullTextLucene.FullTextTrigger existing = new FullTextLucene.FullTextTrigger();
         existing.init(conn, schema, null, table, false, Trigger.INSERT);
-        String sql = "SELECT * FROM " + StringUtils.quoteIdentifier(schema) +
-                "." + StringUtils.quoteIdentifier(table);
+        String sql = "SELECT * FROM " + StringUtils.quoteIdentifier(schema)
+                + "." + StringUtils.quoteIdentifier(table);
         ResultSet rs = conn.createStatement().executeQuery(sql);
         int columnCount = rs.getMetaData().getColumnCount();
         while (rs.next()) {
@@ -373,10 +362,7 @@ public class FullTextLucene extends FullText {
 
     private static void removeIndexFiles(Connection conn) throws SQLException {
         String path = getIndexPath(conn);
-        IndexAccess access = INDEX_ACCESS.get(path);
-        if (access != null) {
-            removeIndexAccess(access, path);
-        }
+        removeIndexAccess(path);
         if (!path.startsWith(IN_MEMORY_PREFIX)) {
             FileUtils.deleteRecursive(path, false);
         }
@@ -386,17 +372,16 @@ public class FullTextLucene extends FullText {
      * Close the index writer and searcher and remove them from the index access
      * set.
      *
-     * @param access the index writer/searcher wrapper
      * @param indexPath the index path
      */
-    protected static void removeIndexAccess(IndexAccess access, String indexPath)
+    protected static void removeIndexAccess(String indexPath)
             throws SQLException {
         synchronized (INDEX_ACCESS) {
             try {
-                INDEX_ACCESS.remove(indexPath);
-                access.searcher.close();
-                access.reader.close();
-                access.writer.close();
+                IndexAccess access = INDEX_ACCESS.remove(indexPath);
+                if(access != null) {
+                    access.close();
+                }
             } catch (Exception e) {
                 throw convertException(e);
             }
@@ -426,49 +411,50 @@ public class FullTextLucene extends FullText {
         try {
             IndexAccess access = getIndexAccess(conn);
             // take a reference as the searcher may change
-            IndexSearcher searcher = access.searcher;
-            // reuse the same analyzer; it's thread-safe;
-            // also allows subclasses to control the analyzer used.
-            Analyzer analyzer = access.writer.getAnalyzer();
-            QueryParser parser = new QueryParser(Version.LUCENE_30,
-                    LUCENE_FIELD_DATA, analyzer);
-            Query query = parser.parse(text);
-            // Lucene 3 insists on a hard limit and will not provide
-            // a total hits value. Take at least 100 which is
-            // an optimal limit for Lucene as any more
-            // will trigger writing results to disk.
-            int maxResults = (limit == 0 ? 100 : limit) + offset;
-            TopDocs docs = searcher.search(query, maxResults);
-            if (limit == 0) {
-                limit = docs.totalHits;
-            }
-            for (int i = 0, len = docs.scoreDocs.length;
-                    i < limit && i + offset < docs.totalHits
-                    && i + offset < len; i++) {
-                ScoreDoc sd = docs.scoreDocs[i + offset];
-                Document doc = searcher.doc(sd.doc);
-                float score = sd.score;
-                String q = doc.get(LUCENE_FIELD_QUERY);
-                if (data) {
-                    int idx = q.indexOf(" WHERE ");
-                    JdbcConnection c = (JdbcConnection) conn;
-                    Session session = (Session) c.getSession();
-                    Parser p = new Parser(session);
-                    String tab = q.substring(0, idx);
-                    ExpressionColumn expr = (ExpressionColumn) p.parseExpression(tab);
-                    String schemaName = expr.getOriginalTableAliasName();
-                    String tableName = expr.getColumnName();
-                    q = q.substring(idx + " WHERE ".length());
-                    Object[][] columnData = parseKey(conn, q);
-                    result.addRow(
-                            schemaName,
-                            tableName,
-                            columnData[0],
-                            columnData[1],
-                            score);
-                } else {
-                    result.addRow(q, score);
+            IndexSearcher searcher = access.getSearcher();
+            try {
+                // reuse the same analyzer; it's thread-safe;
+                // also allows subclasses to control the analyzer used.
+                Analyzer analyzer = access.writer.getAnalyzer();
+                QueryParser parser = new QueryParser(Version.LUCENE_30,
+                        LUCENE_FIELD_DATA, analyzer);
+                Query query = parser.parse(text);
+                // Lucene 3 insists on a hard limit and will not provide
+                // a total hits value. Take at least 100 which is
+                // an optimal limit for Lucene as any more
+                // will trigger writing results to disk.
+                int maxResults = (limit == 0 ? 100 : limit) + offset;
+                TopDocs docs = searcher.search(query, maxResults);
+                if (limit == 0) {
+                    limit = docs.totalHits;
                 }
+                for (int i = 0, len = docs.scoreDocs.length; i < limit
+                        && i + offset < docs.totalHits
+                        && i + offset < len; i++) {
+                    ScoreDoc sd = docs.scoreDocs[i + offset];
+                    Document doc = searcher.doc(sd.doc);
+                    float score = sd.score;
+                    String q = doc.get(LUCENE_FIELD_QUERY);
+                    if (data) {
+                        int idx = q.indexOf(" WHERE ");
+                        JdbcConnection c = (JdbcConnection) conn;
+                        Session session = (Session) c.getSession();
+                        Parser p = new Parser(session);
+                        String tab = q.substring(0, idx);
+                        ExpressionColumn expr = (ExpressionColumn) p
+                                .parseExpression(tab);
+                        String schemaName = expr.getOriginalTableAliasName();
+                        String tableName = expr.getColumnName();
+                        q = q.substring(idx + " WHERE ".length());
+                        Object[][] columnData = parseKey(conn, q);
+                        result.addRow(schemaName, tableName, columnData[0],
+                                columnData[1], score);
+                    } else {
+                        result.addRow(q, score);
+                    }
+                }
+            } finally {
+                access.returnSearcher(searcher);
             }
         } catch (Exception e) {
             throw convertException(e);
@@ -479,16 +465,16 @@ public class FullTextLucene extends FullText {
     /**
      * Trigger updates the index when a inserting, updating, or deleting a row.
      */
-    public static class FullTextTrigger implements Trigger {
+    public static final class FullTextTrigger implements Trigger {
 
-        protected String schema;
-        protected String table;
-        protected int[] keys;
-        protected int[] indexColumns;
-        protected String[] columns;
-        protected int[] columnTypes;
-        protected String indexPath;
-        protected IndexAccess indexAccess;
+        private String schema;
+        private String table;
+        private int[] keys;
+        private int[] indexColumns;
+        private String[] columns;
+        private int[] columnTypes;
+        private String indexPath;
+        private IndexAccess indexAccess;
 
         /**
          * INTERNAL
@@ -511,8 +497,7 @@ public class FullTextLucene extends FullText {
                 columnList.add(rs.getString("COLUMN_NAME"));
             }
             columnTypes = new int[columnList.size()];
-            columns = new String[columnList.size()];
-            columnList.toArray(columns);
+            columns = columnList.toArray(new String[0]);
             rs = meta.getColumns(null,
                     StringUtils.escapeMetaDataPattern(schemaName),
                     StringUtils.escapeMetaDataPattern(tableName),
@@ -520,7 +505,7 @@ public class FullTextLucene extends FullText {
             for (int i = 0; rs.next(); i++) {
                 columnTypes[i] = rs.getInt("DATA_TYPE");
             }
-            if (keyList.size() == 0) {
+            if (keyList.isEmpty()) {
                 rs = meta.getPrimaryKeys(null,
                         StringUtils.escapeMetaDataPattern(schemaName),
                         tableName);
@@ -528,7 +513,7 @@ public class FullTextLucene extends FullText {
                     keyList.add(rs.getString("COLUMN_NAME"));
                 }
             }
-            if (keyList.size() == 0) {
+            if (keyList.isEmpty()) {
                 throw throwException("No primary key for table " + tableName);
             }
             ArrayList<String> indexList = New.arrayList();
@@ -541,12 +526,11 @@ public class FullTextLucene extends FullText {
             if (rs.next()) {
                 String cols = rs.getString(1);
                 if (cols != null) {
-                    for (String s : StringUtils.arraySplit(cols, ',', true)) {
-                        indexList.add(s);
-                    }
+                    Collections.addAll(indexList,
+                            StringUtils.arraySplit(cols, ',', true));
                 }
             }
-            if (indexList.size() == 0) {
+            if (indexList.isEmpty()) {
                 indexList.addAll(columnList);
             }
             keys = new int[keyList.size()];
@@ -583,10 +567,7 @@ public class FullTextLucene extends FullText {
          */
         @Override
         public void close() throws SQLException {
-            if (indexAccess != null) {
-                removeIndexAccess(indexAccess, indexPath);
-                indexAccess = null;
-            }
+            removeIndexAccess(indexPath);
         }
 
         /**
@@ -602,12 +583,7 @@ public class FullTextLucene extends FullText {
          */
         void commitIndex() throws SQLException {
             try {
-                indexAccess.writer.commit();
-                // recreate Searcher with the IndexWriter's reader.
-                indexAccess.searcher.close();
-                indexAccess.reader.close();
-                indexAccess.reader = IndexReader.open(indexAccess.writer, true);
-                indexAccess.searcher = new IndexSearcher(indexAccess.reader);
+                indexAccess.commit();
             } catch (IOException e) {
                 throw convertException(e);
             }
@@ -699,22 +675,96 @@ public class FullTextLucene extends FullText {
     /**
      * A wrapper for the Lucene writer and searcher.
      */
-    static class IndexAccess {
+    static final class IndexAccess {
 
         /**
          * The index writer.
          */
-        IndexWriter writer;
+        final IndexWriter writer;
 
         /**
-         * The index reader.
+         * Map of usage counters for outstanding searchers.
          */
-        IndexReader reader;
+        private final Map<IndexSearcher,Integer> counters = new HashMap<>();
+
+        /**
+         * Usage counter for current searcher.
+         */
+        private int counter;
 
         /**
          * The index searcher.
          */
-        IndexSearcher searcher;
-    }
+        private IndexSearcher searcher;
 
+        IndexAccess(IndexWriter writer) throws IOException {
+            this.writer = writer;
+            IndexReader reader = IndexReader.open(writer, true);
+            searcher = new IndexSearcher(reader);
+        }
+
+        /**
+         * Start using the searcher.
+         *
+         * @return the searcher
+         */
+        synchronized IndexSearcher getSearcher() {
+            ++counter;
+            return searcher;
+        }
+
+        /**
+         * Stop using the searcher.
+         *
+         * @param searcher the searcher
+         */
+        synchronized void returnSearcher(IndexSearcher searcher) {
+            if (this.searcher == searcher) {
+                --counter;
+                assert counter >= 0;
+            } else {
+                Integer cnt = counters.remove(searcher);
+                assert cnt != null;
+                if(--cnt == 0) {
+                    closeSearcher(searcher);
+                } else {
+                    counters.put(searcher, cnt);
+                }
+            }
+        }
+
+        /**
+         * Commit the changes.
+         */
+        public synchronized void commit() throws IOException {
+            writer.commit();
+            if (counter != 0) {
+                counters.put(searcher, counter);
+                counter = 0;
+            } else {
+                closeSearcher(searcher);
+            }
+            // recreate Searcher with the IndexWriter's reader.
+            searcher = new IndexSearcher(IndexReader.open(writer, true));
+        }
+
+        /**
+         * Close the index.
+         */
+        public synchronized void close() throws IOException {
+            for (IndexSearcher searcher : counters.keySet()) {
+                closeSearcher(searcher);
+            }
+            counters.clear();
+            closeSearcher(searcher);
+            searcher = null;
+            writer.close();
+        }
+
+        private static void closeSearcher(IndexSearcher searcher) {
+            IndexReader indexReader = searcher.getIndexReader();
+            try { searcher.close(); } catch(IOException ignore) {/**/}
+            try { indexReader.close(); } catch(IOException ignore) {/**/}
+        }
+    }
 }
