@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -17,7 +17,7 @@ import org.h2.value.ValueLong;
  */
 class RangeCursor implements Cursor {
 
-    private Session session;
+    private final Session session;
     private boolean beforeFirst;
     private long current;
     private Row currentRow;
