@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -21,6 +21,7 @@ import org.h2.api.ErrorCode;
 import org.h2.engine.Constants;
 import org.h2.message.DbException;
 import org.h2.message.TraceObject;
+import org.h2.store.RangeReader;
 import org.h2.util.IOUtils;
 import org.h2.util.Task;
 import org.h2.value.Value;
@@ -137,7 +138,7 @@ public class JdbcClob extends TraceObject implements NClob
             if (value.getPrecision() != 0) {
                 throw DbException.getInvalidValueException("length", value.getPrecision());
             }
-            final JdbcConnection c = conn;
+            final JdbcConnection c = conn; // required to avoid synthetic method creation
             // PipedReader / PipedWriter are a lot slower
             // than PipedInputStream / PipedOutputStream
             // (Sun/Oracle Java 1.6.0_20)
@@ -227,12 +228,34 @@ public class JdbcClob extends TraceObject implements NClob
     }
 
     /**
-     * [Not supported] Sets a substring.
+     * Fills the Clob. This is only supported for new, empty Clob objects that
+     * were created with Connection.createClob() or createNClob(). The position
+     * must be 1, meaning the whole Clob data is set.
+     *
+     * @param pos where to start writing (the first character is at position 1)
+     * @param str the string to add
+     * @param offset the string offset
+     * @param len the number of characters to read
+     * @return the length of the added text
      */
     @Override
     public int setString(long pos, String str, int offset, int len)
             throws SQLException {
-        throw unsupported("LOB update");
+        try {
+            if (isDebugEnabled()) {
+                debugCode("setString(" + pos + ", " + quote(str) + ", " + offset + ", " + len + ");");
+            }
+            checkClosed();
+            if (pos != 1) {
+                throw DbException.getInvalidValueException("pos", pos);
+            } else if (str == null) {
+                throw DbException.getInvalidValueException("str", str);
+            }
+            value = conn.createClob(new RangeReader(new StringReader(str), offset, len), -1);
+            return (int) value.getPrecision();
+        } catch (Exception e) {
+            throw logAndConvert(e);
+        }
     }
 
     /**
@@ -261,11 +284,21 @@ public class JdbcClob extends TraceObject implements NClob
     }
 
     /**
-     * [Not supported] Returns the reader, starting from an offset.
+     * Returns the reader, starting from an offset.
+     *
+     * @param pos 1-based offset
+     * @param length length of requested area
+     * @return the reader
      */
     @Override
     public Reader getCharacterStream(long pos, long length) throws SQLException {
-        throw unsupported("LOB subset");
+        try {
+            debugCodeCall("getCharacterStream(pos, length)");
+            checkClosed();
+            return value.getReader(pos, length);
+        } catch (Exception e) {
+            throw logAndConvert(e);
+        }
     }
 
     private void checkClosed() {
