@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -82,13 +82,21 @@ public class CacheLongKeyLIRS<V> {
      * Remove all entries.
      */
     public void clear() {
-        long max = Math.max(1, maxMemory / segmentCount);
+        long max = getMaxItemSize();
         for (int i = 0; i < segmentCount; i++) {
-            segments[i] = new Segment<V>(
+            segments[i] = new Segment<>(
                     max, stackMoveDistance, 8, nonResidentQueueSize);
         }
     }
 
+    /**
+     * Determines max size of the data item size to fit into cache
+     * @return data items size limit
+     */
+    public long getMaxItemSize() {
+        return Math.max(1, maxMemory / segmentCount);
+    }
+
     private Entry<V> find(long key) {
         int hash = getHash(key);
         return getSegment(hash).find(key, hash);
@@ -162,7 +170,7 @@ public class CacheLongKeyLIRS<V> {
         Segment<V> s2 = segments[segmentIndex];
         if (s == s2) {
             // no other thread resized, so we do
-            s = new Segment<V>(s, newLen);
+            s = new Segment<>(s, newLen);
             segments[segmentIndex] = s;
         }
         return s;
@@ -296,7 +304,7 @@ public class CacheLongKeyLIRS<V> {
      * @return the entry set
      */
     public synchronized Set<Map.Entry<Long, V>> entrySet() {
-        HashMap<Long, V> map = new HashMap<Long, V>();
+        HashMap<Long, V> map = new HashMap<>();
         for (long k : keySet()) {
             map.put(k,  find(k).value);
         }
@@ -309,7 +317,7 @@ public class CacheLongKeyLIRS<V> {
      * @return the set of keys
      */
     public Set<Long> keySet() {
-        HashSet<Long> set = new HashSet<Long>();
+        HashSet<Long> set = new HashSet<>();
         for (Segment<V> s : segments) {
             set.addAll(s.keySet());
         }
@@ -403,7 +411,7 @@ public class CacheLongKeyLIRS<V> {
      * @return the key list
      */
     public List<Long> keys(boolean cold, boolean nonResident) {
-        ArrayList<Long> keys = new ArrayList<Long>();
+        ArrayList<Long> keys = new ArrayList<>();
         for (Segment<V> s : segments) {
             keys.addAll(s.keys(cold, nonResident));
         }
@@ -416,7 +424,7 @@ public class CacheLongKeyLIRS<V> {
      * @return the entry set
      */
     public List<V> values() {
-        ArrayList<V> list = new ArrayList<V>();
+        ArrayList<V> list = new ArrayList<>();
         for (long k : keySet()) {
             V value = find(k).value;
             if (value != null) {
@@ -451,7 +459,7 @@ public class CacheLongKeyLIRS<V> {
      * @return the map
      */
     public Map<Long, V> getMap() {
-        HashMap<Long, V> map = new HashMap<Long, V>();
+        HashMap<Long, V> map = new HashMap<>();
         for (long k : keySet()) {
             V x = find(k).value;
             if (x != null) {
@@ -591,11 +599,11 @@ public class CacheLongKeyLIRS<V> {
             mask = len - 1;
 
             // initialize the stack and queue heads
-            stack = new Entry<V>();
+            stack = new Entry<>();
             stack.stackPrev = stack.stackNext = stack;
-            queue = new Entry<V>();
+            queue = new Entry<>();
             queue.queuePrev = queue.queueNext = queue;
-            queue2 = new Entry<V>();
+            queue2 = new Entry<>();
             queue2.queuePrev = queue2.queueNext = queue2;
 
             @SuppressWarnings("unchecked")
@@ -671,7 +679,7 @@ public class CacheLongKeyLIRS<V> {
         }
 
         private static <V> Entry<V> copy(Entry<V> old) {
-            Entry<V> e = new Entry<V>();
+            Entry<V> e = new Entry<>();
             e.key = old.key;
             e.value = old.value;
             e.memory = old.memory;
@@ -805,7 +813,7 @@ public class CacheLongKeyLIRS<V> {
                 // the new entry is too big to fit
                 return old;
             }
-            e = new Entry<V>();
+            e = new Entry<>();
             e.key = key;
             e.value = value;
             e.memory = memory;
@@ -1033,7 +1041,7 @@ public class CacheLongKeyLIRS<V> {
          * @return the key list
          */
         synchronized List<Long> keys(boolean cold, boolean nonResident) {
-            ArrayList<Long> keys = new ArrayList<Long>();
+            ArrayList<Long> keys = new ArrayList<>();
             if (cold) {
                 Entry<V> start = nonResident ? queue2 : queue;
                 for (Entry<V> e = start.queueNext; e != start;
@@ -1068,7 +1076,7 @@ public class CacheLongKeyLIRS<V> {
          * @return the set of keys
          */
         synchronized Set<Long> keySet() {
-            HashSet<Long> set = new HashSet<Long>();
+            HashSet<Long> set = new HashSet<>();
             for (Entry<V> e = stack.stackNext; e != stack; e = e.stackNext) {
                 set.add(e.key);
             }
@@ -1184,7 +1192,7 @@ public class CacheLongKeyLIRS<V> {
          * The number of entries in the non-resident queue, as a factor of the
          * number of all other entries in the map.
          */
-        public int nonResidentQueueSize = 3;
+        public final int nonResidentQueueSize = 3;
 
     }
 
