@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -44,7 +44,7 @@ public class MVRTreeMap<V> extends MVMap<SpatialKey, V> {
      * @return the map
      */
     public static <V> MVRTreeMap<V> create(int dimensions, DataType valueType) {
-        return new MVRTreeMap<V>(dimensions, valueType);
+        return new MVRTreeMap<>(dimensions, valueType);
     }
 
     @Override
@@ -612,7 +612,7 @@ public class MVRTreeMap<V> extends MVMap<SpatialKey, V> {
             if (valueType == null) {
                 valueType = new ObjectDataType();
             }
-            return new MVRTreeMap<V>(dimensions, valueType);
+            return new MVRTreeMap<>(dimensions, valueType);
         }
 
     }
