@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -10,6 +10,7 @@ import java.util.HashSet;
 import java.util.Map;
 import java.util.concurrent.ConcurrentHashMap;
 import org.h2.api.ErrorCode;
+import org.h2.command.ddl.CreateSynonymData;
 import org.h2.command.ddl.CreateTableData;
 import org.h2.constraint.Constraint;
 import org.h2.engine.Database;
@@ -27,6 +28,7 @@ import org.h2.mvstore.db.MVTableEngine;
 import org.h2.table.RegularTable;
 import org.h2.table.Table;
 import org.h2.table.TableLink;
+import org.h2.table.TableSynonym;
 import org.h2.util.New;
 import org.h2.util.StringUtils;
 
@@ -41,6 +43,7 @@ public class Schema extends DbObjectBase {
     private ArrayList<String> tableEngineParams;
 
     private final ConcurrentHashMap<String, Table> tablesAndViews;
+    private final ConcurrentHashMap<String, TableSynonym> synonyms;
     private final ConcurrentHashMap<String, Index> indexes;
     private final ConcurrentHashMap<String, Sequence> sequences;
     private final ConcurrentHashMap<String, TriggerObject> triggers;
@@ -53,7 +56,7 @@ public class Schema extends DbObjectBase {
      * avoid returning the same unique name twice when multiple threads
      * concurrently create objects.
      */
-    private final HashSet<String> temporaryUniqueNames = New.hashSet();
+    private final HashSet<String> temporaryUniqueNames = new HashSet<>();
 
     /**
      * Create a new schema object.
@@ -68,6 +71,7 @@ public class Schema extends DbObjectBase {
     public Schema(Database database, int id, String schemaName, User owner,
             boolean system) {
         tablesAndViews = database.newConcurrentStringMap();
+        synonyms = database.newConcurrentStringMap();
         indexes = database.newConcurrentStringMap();
         sequences = database.newConcurrentStringMap();
         triggers = database.newConcurrentStringMap();
@@ -112,6 +116,16 @@ public class Schema extends DbObjectBase {
         return DbObject.SCHEMA;
     }
 
+    /**
+     * Return whether is this schema is empty (does not contain any objects).
+     *
+     * @return {@code true} if this schema is empty, {@code false} otherwise
+     */
+    public boolean isEmpty() {
+        return tablesAndViews.isEmpty() && synonyms.isEmpty() && indexes.isEmpty() && sequences.isEmpty()
+                && triggers.isEmpty() && constraints.isEmpty() && constants.isEmpty() && functions.isEmpty();
+    }
+
     @Override
     public void removeChildrenAndResources(Session session) {
         while (triggers != null && triggers.size() > 0) {
@@ -129,7 +143,7 @@ public class Schema extends DbObjectBase {
             runLoopAgain = false;
             if (tablesAndViews != null) {
                 // Loop over a copy because the map is modified underneath us.
-                for (Table obj : New.arrayList(tablesAndViews.values())) {
+                for (Table obj : new ArrayList<>(tablesAndViews.values())) {
                     // Check for null because multiple tables might be deleted
                     // in one go underneath us.
                     if (obj.getName() != null) {
@@ -201,6 +215,9 @@ public class Schema extends DbObjectBase {
         case DbObject.TABLE_OR_VIEW:
             result = tablesAndViews;
             break;
+        case DbObject.SYNONYM:
+            result = synonyms;
+            break;
         case DbObject.SEQUENCE:
             result = sequences;
             break;
@@ -273,7 +290,7 @@ public class Schema extends DbObjectBase {
     /**
      * Try to find a table or view with this name. This method returns null if
      * no object with this name exists. Local temporary tables are also
-     * returned.
+     * returned. Synonyms are not returned or resolved.
      *
      * @param session the session
      * @param name the object name
@@ -287,6 +304,39 @@ public class Schema extends DbObjectBase {
         return table;
     }
 
+    /**
+     * Try to find a table or view with this name. This method returns null if
+     * no object with this name exists. Local temporary tables are also
+     * returned. If a synonym with this name exists, the backing table of the
+     * synonym is returned
+     *
+     * @param session the session
+     * @param name the object name
+     * @return the object or null
+     */
+    public Table resolveTableOrView(Session session, String name) {
+        Table table = findTableOrView(session, name);
+        if (table == null) {
+            TableSynonym synonym = synonyms.get(name);
+            if (synonym != null) {
+                return synonym.getSynonymFor();
+            }
+            return null;
+        }
+        return table;
+    }
+
+    /**
+     * Try to find a synonym with this name. This method returns null if
+     * no object with this name exists.
+     *
+     * @param name the object name
+     * @return the object or null
+     */
+    public TableSynonym getSynonym(String name) {
+        return synonyms.get(name);
+    }
+
     /**
      * Try to find an index with this name. This method returns null if
      * no object with this name exists.
@@ -527,6 +577,7 @@ public class Schema extends DbObjectBase {
     public ArrayList<SchemaObject> getAll() {
         ArrayList<SchemaObject> all = New.arrayList();
         all.addAll(getMap(DbObject.TABLE_OR_VIEW).values());
+        all.addAll(getMap(DbObject.SYNONYM).values());
         all.addAll(getMap(DbObject.SEQUENCE).values());
         all.addAll(getMap(DbObject.INDEX).values());
         all.addAll(getMap(DbObject.TRIGGER).values());
@@ -544,7 +595,7 @@ public class Schema extends DbObjectBase {
      */
     public ArrayList<SchemaObject> getAll(int type) {
         Map<String, SchemaObject> map = getMap(type);
-        return New.arrayList(map.values());
+        return new ArrayList<>(map.values());
     }
 
     /**
@@ -554,7 +605,14 @@ public class Schema extends DbObjectBase {
      */
     public ArrayList<Table> getAllTablesAndViews() {
         synchronized (database) {
-            return New.arrayList(tablesAndViews.values());
+            return new ArrayList<>(tablesAndViews.values());
+        }
+    }
+
+
+    public ArrayList<TableSynonym> getAllSynonyms() {
+        synchronized (database) {
+            return new ArrayList<>(synonyms.values());
         }
     }
 
@@ -615,6 +673,20 @@ public class Schema extends DbObjectBase {
         }
     }
 
+    /**
+     * Add a table synonym to the schema.
+     *
+     * @param data the create synonym information
+     * @return the created {@link TableSynonym} object
+     */
+    public TableSynonym createSynonym(CreateSynonymData data) {
+        synchronized (database) {
+            database.lockMeta(data.session);
+            data.schema = this;
+            return new TableSynonym(data);
+        }
+    }
+
     /**
      * Add a linked table to the schema.
      *
