@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -12,6 +12,8 @@ import java.nio.ByteBuffer;
 import java.nio.CharBuffer;
 import java.nio.charset.CharsetEncoder;
 import java.nio.charset.CodingErrorAction;
+import java.nio.charset.StandardCharsets;
+
 import org.h2.engine.Constants;
 
 /**
@@ -25,7 +27,7 @@ public class CountingReaderInputStream extends InputStream {
     private final CharBuffer charBuffer =
             CharBuffer.allocate(Constants.IO_BUFFER_SIZE);
 
-    private final CharsetEncoder encoder = Constants.UTF8.newEncoder().
+    private final CharsetEncoder encoder = StandardCharsets.UTF_8.newEncoder().
             onMalformedInput(CodingErrorAction.REPLACE).
             onUnmappableCharacter(CodingErrorAction.REPLACE);
 
