@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  *
@@ -17,14 +17,17 @@ import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Timestamp;
+import java.util.Arrays;
+
 import org.h2.api.ErrorCode;
 import org.h2.engine.Constants;
 import org.h2.engine.SysProperties;
 import org.h2.message.DbException;
-import org.h2.mvstore.DataUtils;
 import org.h2.tools.SimpleResultSet;
+import org.h2.util.Bits;
 import org.h2.util.DateTimeUtils;
 import org.h2.util.MathUtils;
+import org.h2.util.Utils;
 import org.h2.value.DataType;
 import org.h2.value.Value;
 import org.h2.value.ValueArray;
@@ -127,11 +130,7 @@ public class Data {
      * @param x the value
      */
     public void setInt(int pos, int x) {
-        byte[] buff = data;
-        buff[pos] = (byte) (x >> 24);
-        buff[pos + 1] = (byte) (x >> 16);
-        buff[pos + 2] = (byte) (x >> 8);
-        buff[pos + 3] = (byte) x;
+        Bits.writeInt(data, pos, x);
     }
 
     /**
@@ -141,11 +140,7 @@ public class Data {
      * @param x the value
      */
     public void writeInt(int x) {
-        byte[] buff = data;
-        buff[pos] = (byte) (x >> 24);
-        buff[pos + 1] = (byte) (x >> 16);
-        buff[pos + 2] = (byte) (x >> 8);
-        buff[pos + 3] = (byte) x;
+        Bits.writeInt(data, pos, x);
         pos += 4;
     }
 
@@ -156,11 +151,7 @@ public class Data {
      * @return the value
      */
     public int readInt() {
-        byte[] buff = data;
-        int x = (buff[pos] << 24) +
-                ((buff[pos+1] & 0xff) << 16) +
-                ((buff[pos+2] & 0xff) << 8) +
-                (buff[pos+3] & 0xff);
+        int x = Bits.readInt(data, pos);
         pos += 4;
         return x;
     }
@@ -400,7 +391,9 @@ public class Data {
      * @return the long value
      */
     public long readLong() {
-        return ((long) (readInt()) << 32) + (readInt() & 0xffffffffL);
+        long x = Bits.readLong(data, pos);
+        pos += 8;
+        return x;
     }
 
     /**
@@ -409,8 +402,8 @@ public class Data {
      * @param x the value
      */
     public void writeLong(long x) {
-        writeInt((int) (x >>> 32));
-        writeInt((int) x);
+        Bits.writeLong(data, pos, x);
+        pos += 8;
     }
 
     /**
@@ -427,8 +420,7 @@ public class Data {
         int type = v.getType();
         switch (type) {
         case Value.BOOLEAN:
-            writeByte((byte) (v.getBoolean().booleanValue() ?
-                    BOOLEAN_TRUE : BOOLEAN_FALSE));
+            writeByte((byte) (v.getBoolean() ? BOOLEAN_TRUE : BOOLEAN_FALSE));
             break;
         case Value.BYTE:
             writeByte((byte) type);
@@ -438,6 +430,7 @@ public class Data {
             writeByte((byte) type);
             writeShortInt(v.getShort());
             break;
+        case Value.ENUM:
         case Value.INT: {
             int x = v.getInt();
             if (x < 0) {
@@ -498,8 +491,8 @@ public class Data {
                 writeByte((byte) LOCAL_TIME);
                 ValueTime t = (ValueTime) v;
                 long nanos = t.getNanos();
-                long millis = nanos / 1000000;
-                nanos -= millis * 1000000;
+                long millis = nanos / 1_000_000;
+                nanos -= millis * 1_000_000;
                 writeVarLong(millis);
                 writeVarLong(nanos);
             } else {
@@ -526,15 +519,15 @@ public class Data {
                 long dateValue = ts.getDateValue();
                 writeVarLong(dateValue);
                 long nanos = ts.getTimeNanos();
-                long millis = nanos / 1000000;
-                nanos -= millis * 1000000;
+                long millis = nanos / 1_000_000;
+                nanos -= millis * 1_000_000;
                 writeVarLong(millis);
                 writeVarLong(nanos);
             } else {
                 Timestamp ts = v.getTimestamp();
                 writeByte((byte) type);
                 writeVarLong(DateTimeUtils.getTimeLocalWithoutDst(ts));
-                writeVarInt(ts.getNanos() % 1000000);
+                writeVarInt(ts.getNanos() % 1_000_000);
             }
             break;
         }
@@ -723,11 +716,12 @@ public class Data {
         case Value.NULL:
             return ValueNull.INSTANCE;
         case BOOLEAN_TRUE:
-            return ValueBoolean.get(true);
+            return ValueBoolean.TRUE;
         case BOOLEAN_FALSE:
-            return ValueBoolean.get(false);
+            return ValueBoolean.FALSE;
         case INT_NEG:
             return ValueInt.get(-readVarInt());
+        case Value.ENUM:
         case Value.INT:
             return ValueInt.get(readVarInt());
         case LONG_NEG:
@@ -751,7 +745,7 @@ public class Data {
         case Value.DECIMAL: {
             int scale = readVarInt();
             int len = readVarInt();
-            byte[] buff = DataUtils.newBytes(len);
+            byte[] buff = Utils.newBytes(len);
             read(buff, 0, len);
             BigInteger b = new BigInteger(buff);
             return ValueDecimal.get(new BigDecimal(b, scale));
@@ -764,7 +758,7 @@ public class Data {
             return ValueDate.fromMillis(DateTimeUtils.getTimeUTCWithoutDst(x));
         }
         case LOCAL_TIME: {
-            long nanos = readVarLong() * 1000000 + readVarLong();
+            long nanos = readVarLong() * 1_000_000 + readVarLong();
             return ValueTime.fromNanos(nanos);
         }
         case Value.TIME:
@@ -773,7 +767,7 @@ public class Data {
                     DateTimeUtils.getTimeUTCWithoutDst(readVarLong()));
         case LOCAL_TIMESTAMP: {
             long dateValue = readVarLong();
-            long nanos = readVarLong() * 1000000 + readVarLong();
+            long nanos = readVarLong() * 1_000_000 + readVarLong();
             return ValueTimestamp.fromDateValueAndNanos(dateValue, nanos);
         }
         case Value.TIMESTAMP: {
@@ -789,19 +783,19 @@ public class Data {
         }
         case Value.BYTES: {
             int len = readVarInt();
-            byte[] b = DataUtils.newBytes(len);
+            byte[] b = Utils.newBytes(len);
             read(b, 0, len);
             return ValueBytes.getNoCopy(b);
         }
         case Value.GEOMETRY: {
             int len = readVarInt();
-            byte[] b = DataUtils.newBytes(len);
+            byte[] b = Utils.newBytes(len);
             read(b, 0, len);
             return ValueGeometry.get(b);
         }
         case Value.JAVA_OBJECT: {
             int len = readVarInt();
-            byte[] b = DataUtils.newBytes(len);
+            byte[] b = Utils.newBytes(len);
             read(b, 0, len);
             return ValueJavaObject.getNoCopy(null, b, handler);
         }
@@ -831,16 +825,15 @@ public class Data {
         case Value.CLOB: {
             int smallLen = readVarInt();
             if (smallLen >= 0) {
-                byte[] small = DataUtils.newBytes(smallLen);
+                byte[] small = Utils.newBytes(smallLen);
                 read(small, 0, smallLen);
                 return ValueLobDb.createSmallLob(type, small);
             } else if (smallLen == -3) {
                 int tableId = readVarInt();
                 long lobId = readVarLong();
                 long precision = readVarLong();
-                ValueLobDb lob = ValueLobDb.create(type, handler, tableId,
+                return ValueLobDb.create(type, handler, tableId,
                         lobId, null, precision);
-                return lob;
             } else {
                 int tableId = readVarInt();
                 int objectId = readVarInt();
@@ -876,10 +869,7 @@ public class Data {
             for (int i = 0; i < columns; i++) {
                 rs.addColumn(readString(), readVarInt(), readVarInt(), readVarInt());
             }
-            while (true) {
-                if (readByte() == 0) {
-                    break;
-                }
+            while (readByte() != 0) {
                 Object[] o = new Object[columns];
                 for (int i = 0; i < columns; i++) {
                     o[i] = readValue().getObject();
@@ -895,7 +885,7 @@ public class Data {
                 return ValueLong.get(type - LONG_0_7);
             } else if (type >= BYTES_0_31 && type < BYTES_0_31 + 32) {
                 int len = type - BYTES_0_31;
-                byte[] b = DataUtils.newBytes(len);
+                byte[] b = Utils.newBytes(len);
                 read(b, 0, len);
                 return ValueBytes.getNoCopy(b);
             } else if (type >= STRING_0_31 && type < STRING_0_31 + 32) {
@@ -933,6 +923,7 @@ public class Data {
             return 2;
         case Value.SHORT:
             return 3;
+        case Value.ENUM:
         case Value.INT: {
             int x = v.getInt();
             if (x < 0) {
@@ -1008,8 +999,8 @@ public class Data {
         case Value.TIME:
             if (STORE_LOCAL_TIME) {
                 long nanos = ((ValueTime) v).getNanos();
-                long millis = nanos / 1000000;
-                nanos -= millis * 1000000;
+                long millis = nanos / 1_000_000;
+                nanos -= millis * 1_000_000;
                 return 1 + getVarLongLen(millis) + getVarLongLen(nanos);
             }
             return 1 + getVarLongLen(DateTimeUtils.getTimeLocalWithoutDst(v.getTime()));
@@ -1026,14 +1017,14 @@ public class Data {
                 ValueTimestamp ts = (ValueTimestamp) v;
                 long dateValue = ts.getDateValue();
                 long nanos = ts.getTimeNanos();
-                long millis = nanos / 1000000;
-                nanos -= millis * 1000000;
+                long millis = nanos / 1_000_000;
+                nanos -= millis * 1_000_000;
                 return 1 + getVarLongLen(dateValue) + getVarLongLen(millis) +
                         getVarLongLen(nanos);
             }
             Timestamp ts = v.getTimestamp();
             return 1 + getVarLongLen(DateTimeUtils.getTimeLocalWithoutDst(ts)) +
-                    getVarIntLen(ts.getNanos() % 1000000);
+                    getVarIntLen(ts.getNanos() % 1_000_000);
         }
         case Value.TIMESTAMP_TZ: {
             ValueTimestampTimeZone ts = (ValueTimestampTimeZone) v;
@@ -1178,8 +1169,7 @@ public class Data {
      */
     public void truncate(int size) {
         if (pos > size) {
-            byte[] buff = new byte[size];
-            System.arraycopy(data, 0, buff, 0, size);
+            byte[] buff = Arrays.copyOf(data, size);
             this.pos = size;
             data = buff;
         }
@@ -1320,11 +1310,9 @@ public class Data {
     }
 
     private void expand(int plus) {
-        byte[] d = DataUtils.newBytes((data.length + plus) * 2);
         // must copy everything, because pos could be 0 and data may be
         // still required
-        System.arraycopy(data, 0, d, 0, data.length);
-        data = d;
+        data = Utils.copyBytes(data, (data.length + plus) * 2);
     }
 
     /**
