@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -8,11 +8,11 @@ package org.h2.store;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.Reader;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Map.Entry;
 import org.h2.api.ErrorCode;
-import org.h2.engine.Constants;
 import org.h2.engine.Database;
 import org.h2.message.DbException;
 import org.h2.mvstore.MVMap;
@@ -37,7 +37,7 @@ public class LobStorageMap implements LobStorageInterface {
 
     private boolean init;
 
-    private Object nextLobIdSync = new Object();
+    private final Object nextLobIdSync = new Object();
     private long nextLobId;
 
     /**
@@ -157,7 +157,7 @@ public class LobStorageMap implements LobStorageInterface {
                 return ValueLobDb.createSmallLob(type, small);
             }
             if (maxLength != -1) {
-                in = new LimitInputStream(in, maxLength);
+                in = new RangeInputStream(in, 0L, maxLength);
             }
             return createLob(in, type);
         } catch (IllegalStateException e) {
@@ -182,7 +182,7 @@ public class LobStorageMap implements LobStorageInterface {
                             "len > blobLength, " + len + " > " + maxLength);
                 }
                 byte[] utf8 = new String(small, 0, len)
-                        .getBytes(Constants.UTF8);
+                        .getBytes(StandardCharsets.UTF_8);
                 if (utf8.length > database.getMaxLengthInplaceLob()) {
                     throw new IllegalStateException(
                             "len > maxinplace, " + utf8.length + " > "
@@ -217,9 +217,9 @@ public class LobStorageMap implements LobStorageInterface {
         long lobId = generateLobId();
         long length = streamStore.length(streamStoreId);
         int tableId = LobStorageFrontend.TABLE_TEMP;
-        Object[] value = new Object[] { streamStoreId, tableId, length, 0 };
+        Object[] value = { streamStoreId, tableId, length, 0 };
         lobMap.put(lobId, value);
-        Object[] key = new Object[] { streamStoreId, lobId };
+        Object[] key = { streamStoreId, lobId };
         refMap.put(key, Boolean.TRUE);
         ValueLobDb lob = ValueLobDb.create(
                 type, database, tableId, lobId, null, length);
@@ -259,7 +259,7 @@ public class LobStorageMap implements LobStorageInterface {
         long lobId = generateLobId();
         value[1] = tableId;
         lobMap.put(lobId, value);
-        Object[] key = new Object[] { streamStoreId, lobId };
+        Object[] key = { streamStoreId, lobId };
         refMap.put(key, Boolean.TRUE);
         ValueLobDb lob = ValueLobDb.create(
                 type, database, tableId, lobId, null, length);
@@ -289,19 +289,6 @@ public class LobStorageMap implements LobStorageInterface {
         return streamStore.get(streamStoreId);
     }
 
-    @Override
-    public void setTable(ValueLobDb lob, int tableId) {
-        init();
-        long lobId = lob.getLobId();
-        Object[] value = lobMap.remove(lobId);
-        if (TRACE) {
-            trace("move " + lob.getTableId() + "/" + lob.getLobId() +
-                    " > " + tableId + "/" + lobId);
-        }
-        value[1] = tableId;
-        lobMap.put(lobId, value);
-    }
-
     @Override
     public void removeAllForTable(int tableId) {
         init();
@@ -345,7 +332,7 @@ public class LobStorageMap implements LobStorageInterface {
             return;
         }
         byte[] streamStoreId = (byte[]) value[0];
-        Object[] key = new Object[] {streamStoreId, lobId };
+        Object[] key = {streamStoreId, lobId };
         refMap.remove(key);
         // check if there are more entries for this streamStoreId
         key = new Object[] {streamStoreId, 0L };
