@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -150,7 +150,7 @@ public class PageLog {
      * The session state map.
      * Only used during recovery.
      */
-    private HashMap<Integer, SessionState> sessionStates = New.hashMap();
+    private HashMap<Integer, SessionState> sessionStates = new HashMap<>();
 
     /**
      * The map of pages used by the transaction log.
@@ -869,7 +869,7 @@ public class PageLog {
      * Called after the recovery has been completed.
      */
     void recoverEnd() {
-        sessionStates = New.hashMap();
+        sessionStates = new HashMap<>();
     }
 
     private void flushOut() {
