@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -164,7 +164,7 @@ public class PageStore implements CacheWriter {
     private RegularTable metaTable;
     private PageDataIndex metaIndex;
     private final IntIntHashMap metaRootPageId = new IntIntHashMap();
-    private final HashMap<Integer, PageIndex> metaObjects = New.hashMap();
+    private final HashMap<Integer, PageIndex> metaObjects = new HashMap<>();
     private HashMap<Integer, PageIndex> tempObjects;
 
     /**
@@ -229,7 +229,7 @@ public class PageStore implements CacheWriter {
      * Start collecting statistics.
      */
     public void statisticsStart() {
-        statistics = New.hashMap();
+        statistics = new HashMap<>();
     }
 
     /**
@@ -415,8 +415,8 @@ public class PageStore implements CacheWriter {
     private void writeBack() {
         ArrayList<CacheObject> list = cache.getAllChanged();
         Collections.sort(list);
-        for (int i = 0, size = list.size(); i < size; i++) {
-            writeBack(list.get(i));
+        for (CacheObject cacheObject : list) {
+            writeBack(cacheObject);
         }
     }
 
@@ -1407,7 +1407,7 @@ public class PageStore implements CacheWriter {
         isEmpty &= log.recover(PageLog.RECOVERY_STAGE_REDO);
         boolean setReadOnly = false;
         if (!database.isReadOnly()) {
-            if (log.getInDoubtTransactions().size() == 0) {
+            if (log.getInDoubtTransactions().isEmpty()) {
                 log.recoverEnd();
                 int firstUncommittedSection = getFirstUncommittedSection();
                 log.removeUntil(firstUncommittedSection);
@@ -1421,7 +1421,7 @@ public class PageStore implements CacheWriter {
             if (index.getTable().isTemporary()) {
                 // temporary indexes are removed after opening
                 if (tempObjects == null) {
-                    tempObjects = New.hashMap();
+                    tempObjects = new HashMap<>();
                 }
                 tempObjects.put(index.getId(), index);
             } else {
@@ -1530,7 +1530,7 @@ public class PageStore implements CacheWriter {
         if (tableId == META_TABLE_ID) {
             int rootPageId = row.getValue(3).getInt();
             if (reservedPages == null) {
-                reservedPages = New.hashMap();
+                reservedPages = new HashMap<>();
             }
             reservedPages.put(rootPageId, logPos);
         }
@@ -1722,8 +1722,7 @@ public class PageStore implements CacheWriter {
                     ic.sortType = Integer.parseInt(s);
                     c = c.substring(0, idx);
                 }
-                Column column = tableCols[Integer.parseInt(c)];
-                ic.column = column;
+                ic.column = tableCols[Integer.parseInt(c)];
                 cols[i] = ic;
             }
             IndexType indexType;
@@ -1980,18 +1979,14 @@ public class PageStore implements CacheWriter {
      * @return true if it is correct
      */
     public static boolean checksumTest(byte[] d, int pageId, int pageSize) {
-        int ps = pageSize;
         int s1 = 255 + (d[0] & 255), s2 = 255 + s1;
         s2 += s1 += d[6] & 255;
-        s2 += s1 += d[(ps >> 1) - 1] & 255;
-        s2 += s1 += d[ps >> 1] & 255;
-        s2 += s1 += d[ps - 2] & 255;
-        s2 += s1 += d[ps - 1] & 255;
-        if (d[1] != (byte) (((s1 & 255) + (s1 >> 8)) ^ pageId)
-                || d[2] != (byte) (((s2 & 255) + (s2 >> 8)) ^ (pageId >> 8))) {
-            return false;
-        }
-        return true;
+        s2 += s1 += d[(pageSize >> 1) - 1] & 255;
+        s2 += s1 += d[pageSize >> 1] & 255;
+        s2 += s1 += d[pageSize - 2] & 255;
+        s2 += s1 += d[pageSize - 1] & 255;
+        return d[1] == (byte) (((s1 & 255) + (s1 >> 8)) ^ pageId) && d[2] == (byte) (((s2 & 255) + (s2 >> 8)) ^ (pageId
+                >> 8));
     }
 
     /**
