@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -32,7 +32,7 @@ import org.h2.util.New;
 public class FilePathNioMem extends FilePath {
 
     private static final TreeMap<String, FileNioMemData> MEMORY_FILES =
-            new TreeMap<String, FileNioMemData>();
+            new TreeMap<>();
 
     /**
      * The percentage of uncompressed (cached) entries.
@@ -56,8 +56,7 @@ public class FilePathNioMem extends FilePath {
         synchronized (MEMORY_FILES) {
             if (!atomicReplace && !name.equals(newName.name) &&
                     MEMORY_FILES.containsKey(newName.name)) {
-                throw DbException.get(ErrorCode.FILE_RENAME_FAILED_2,
-                        new String[] { name, newName + " (exists)" });
+                throw DbException.get(ErrorCode.FILE_RENAME_FAILED_2, name, newName + " (exists)");
             }
             FileNioMemData f = getMemoryFile();
             f.setName(newName.name);
@@ -248,8 +247,8 @@ class FilePathNioMemLZF extends FilePathNioMem {
             throw new IllegalArgumentException(path +
                     " doesn't start with " + getScheme());
         }
-        int idx1 = path.indexOf(":");
-        int idx2 = path.lastIndexOf(":");
+        int idx1 = path.indexOf(':');
+        int idx2 = path.lastIndexOf(':');
         final FilePathNioMemLZF p = new FilePathNioMemLZF();
         if (idx1 != -1 && idx1 != idx2) {
             p.compressLaterCachePercent = Float.parseFloat(path.substring(idx1 + 1, idx2));
@@ -260,7 +259,7 @@ class FilePathNioMemLZF extends FilePathNioMem {
 
     @Override
     protected boolean isRoot() {
-        return name.lastIndexOf(":") == name.length() - 1;
+        return name.lastIndexOf(':') == name.length() - 1;
     }
 
     @Override
@@ -386,8 +385,7 @@ class FileNioMem extends FileBase {
             }
         }
 
-        // cast to FileChannel to avoid JDK 1.7 ambiguity
-        FileLock lock = new FileLock((FileChannel) null, position, size, shared) {
+        return new FileLock(new FakeFileChannel(), position, size, shared) {
 
             @Override
             public boolean isValid() {
@@ -399,7 +397,6 @@ class FileNioMem extends FileBase {
                 data.unlock();
             }
         };
-        return lock;
     }
 
     @Override
@@ -444,7 +441,7 @@ class FileNioMemData {
     final int nameHashCode;
 
     private final CompressLaterCache<CompressItem, CompressItem> compressLaterCache =
-        new CompressLaterCache<CompressItem, CompressItem>(CACHE_MIN_SIZE);
+        new CompressLaterCache<>(CACHE_MIN_SIZE);
 
     private String name;
     private final boolean compress;
@@ -682,7 +679,7 @@ class FileNioMemData {
             System.arraycopy(buffers, 0, newBuffers, 0,
                     Math.min(buffers.length, newBuffers.length));
             for (int i = buffers.length; i < blocks; i++) {
-                newBuffers[i] = new AtomicReference<ByteBuffer>(COMPRESSED_EMPTY_BLOCK);
+                newBuffers[i] = new AtomicReference<>(COMPRESSED_EMPTY_BLOCK);
             }
             buffers = newBuffers;
         }
