@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -72,7 +72,7 @@ public final class JoinBatch {
     /**
      * An empty future cursor.
      */
-    static final Future<Cursor> EMPTY_FUTURE_CURSOR = new DoneFuture<Cursor>(EMPTY_CURSOR);
+    static final Future<Cursor> EMPTY_FUTURE_CURSOR = new DoneFuture<>(EMPTY_CURSOR);
 
     /**
      * The top cursor.
@@ -87,7 +87,7 @@ public final class JoinBatch {
     /**
      * The filters.
      */
-    JoinFilter[] filters;
+    final JoinFilter[] filters;
 
     /**
      * Whether this is a batched subquery.
@@ -224,7 +224,7 @@ public final class JoinBatch {
             }
             return false;
         }
-        for (;;) {
+        while (true) {
             if (!found) {
                 if (!batchedNext()) {
                     return false;
@@ -272,7 +272,7 @@ public final class JoinBatch {
             jfId--;
         }
 
-        for (;;) {
+        while (true) {
             fetchCurrent(jfId);
 
             if (!current.isDropped()) {
@@ -359,7 +359,7 @@ public final class JoinBatch {
         assert c != null;
         JoinFilter join = jf.join;
 
-        for (;;) {
+        while (true) {
             if (c == null || !c.next()) {
                 if (newCursor && jf.isOuterJoin()) {
                     // replace cursor with null-row
@@ -694,7 +694,7 @@ public final class JoinBatch {
 
         private boolean full;
 
-        private final List<Future<Cursor>> result = new SingletonList<Future<Cursor>>();
+        private final List<Future<Cursor>> result = new SingletonList<>();
 
         FakeLookupBatch(TableFilter filter) {
             this.filter = filter;
@@ -732,7 +732,7 @@ public final class JoinBatch {
                 return Collections.emptyList();
             }
             Cursor c = filter.getIndex().find(filter, first, last);
-            result.set(0, new DoneFuture<Cursor>(c));
+            result.set(0, new DoneFuture<>(c));
             full = false;
             first = last = null;
             return result;
@@ -1026,8 +1026,8 @@ public final class JoinBatch {
         @Override
         public boolean isBatchFull() {
             // if at least one is full
-            for (int i = 0; i < filters.size(); i++) {
-                if (filters.get(i).isBatchFull()) {
+            for (JoinFilter filter : filters) {
+                if (filter.isBatchFull()) {
                     return true;
                 }
             }
@@ -1118,8 +1118,8 @@ public final class JoinBatch {
             if (joinBatches == null) {
                 return;
             }
-            for (int i = 0, size = joinBatches.size(); i < size; i++) {
-                joinBatches.get(i).viewTopFutureCursor = null;
+            for (JoinBatch joinBatch : joinBatches) {
+                joinBatch.viewTopFutureCursor = null;
             }
         }
     }
