@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -15,11 +15,11 @@ import org.h2.result.SortOrder;
  */
 public class SubQueryInfo {
 
-    private int[] masks;
-    private TableFilter[] filters;
-    private int filter;
-    private SortOrder sortOrder;
-    private SubQueryInfo upper;
+    private final int[] masks;
+    private final TableFilter[] filters;
+    private final int filter;
+    private final SortOrder sortOrder;
+    private final SubQueryInfo upper;
 
     /**
      * @param upper upper level sub-query if any
