@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -243,7 +243,7 @@ public class Shell extends Tool implements Runnable {
                         s = s.replace('\n', ' ').replace('\r', ' ');
                         println("#" + (1 + i) + ": " + s);
                     }
-                    if (history.size() > 0) {
+                    if (!history.isEmpty()) {
                         println("To re-run a statement, type the number and press and enter");
                     } else {
                         println("No history");
@@ -473,7 +473,6 @@ public class Shell extends Tool implements Runnable {
             if (listMode) {
                 e.printStackTrace(err);
             }
-            return;
         }
     }
 
