@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -8,6 +8,7 @@ package org.h2.tools;
 import java.io.InputStream;
 import java.io.Reader;
 import java.math.BigDecimal;
+import java.math.BigInteger;
 import java.net.URL;
 import java.sql.Array;
 import java.sql.Blob;
@@ -28,12 +29,16 @@ import java.sql.Types;
 import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.Map;
+import java.util.UUID;
+
 import org.h2.api.ErrorCode;
 import org.h2.jdbc.JdbcResultSetBackwardsCompat;
 import org.h2.message.DbException;
+import org.h2.util.Bits;
 import org.h2.util.JdbcUtils;
 import org.h2.util.MathUtils;
 import org.h2.util.New;
+import org.h2.util.Utils;
 import org.h2.value.DataType;
 
 /**
@@ -111,7 +116,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
      */
     public void addColumn(String name, int sqlType, String sqlTypeName,
             int precision, int scale) {
-        if (rows != null && rows.size() > 0) {
+        if (rows != null && !rows.isEmpty()) {
             throw new IllegalStateException(
                     "Cannot add a column after adding rows");
         }
@@ -475,10 +480,26 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
     @Override
     public boolean getBoolean(int columnIndex) throws SQLException {
         Object o = get(columnIndex);
-        if (o != null && !(o instanceof Boolean)) {
-            o = Boolean.valueOf(o.toString());
+        if (o == null) {
+            return false;
         }
-        return o == null ? false : ((Boolean) o).booleanValue();
+        if (o instanceof Boolean) {
+            return (Boolean) o;
+        }
+        if (o instanceof Number) {
+            Number n = (Number) o;
+            if (n instanceof Double || n instanceof Float) {
+                return n.doubleValue() != 0;
+            }
+            if (n instanceof BigDecimal) {
+                return ((BigDecimal) n).signum() != 0;
+            }
+            if (n instanceof BigInteger) {
+                return ((BigInteger) n).signum() != 0;
+            }
+            return n.longValue() != 0;
+        }
+        return Utils.parseBoolean(o.toString(), false, true);
     }
 
     /**
@@ -530,6 +551,9 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
         if (o == null || o instanceof byte[]) {
             return (byte[]) o;
         }
+        if (o instanceof UUID) {
+            return Bits.uuidToBytes((UUID) o);
+        }
         return JdbcUtils.serialize(o, null);
     }
 
@@ -819,25 +843,69 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
     }
 
     /**
-     * INTERNAL
+     * Returns the value as an Object of the specified type.
      *
      * @param columnIndex the column index (1, 2, ...)
      * @param type the class of the returned value
+     * @return the value
      */
     @Override
     public <T> T getObject(int columnIndex, Class<T> type) throws SQLException {
-        throw getUnsupportedException();
+        if (wasNull()) {
+            return null;
+        }
+
+        if (type == BigDecimal.class) {
+            return type.cast(getBigDecimal(columnIndex));
+        } else if (type == BigInteger.class) {
+            return type.cast(getBigDecimal(columnIndex).toBigInteger());
+        } else if (type == String.class) {
+            return type.cast(getString(columnIndex));
+        } else if (type == Boolean.class) {
+            return type.cast(getBoolean(columnIndex));
+        } else if (type == Byte.class) {
+            return type.cast(getByte(columnIndex));
+        } else if (type == Short.class) {
+            return type.cast(getShort(columnIndex));
+        } else if (type == Integer.class) {
+            return type.cast(getInt(columnIndex));
+        } else if (type == Long.class) {
+            return type.cast(getLong(columnIndex));
+        } else if (type == Float.class) {
+            return type.cast(getFloat(columnIndex));
+        } else if (type == Double.class) {
+            return type.cast(getDouble(columnIndex));
+        } else if (type == Date.class) {
+            return type.cast(getDate(columnIndex));
+        } else if (type == Time.class) {
+            return type.cast(getTime(columnIndex));
+        } else if (type == Timestamp.class) {
+            return type.cast(getTimestamp(columnIndex));
+        } else if (type == UUID.class) {
+            return type.cast(getObject(columnIndex));
+        } else if (type == byte[].class) {
+            return type.cast(getBytes(columnIndex));
+        } else if (type == java.sql.Array.class) {
+            return type.cast(getArray(columnIndex));
+        } else if (type == Blob.class) {
+            return type.cast(getBlob(columnIndex));
+        } else if (type == Clob.class) {
+            return type.cast(getClob(columnIndex));
+        } else {
+            throw getUnsupportedException();
+        }
     }
 
     /**
-     * INTERNAL
+     * Returns the value as an Object of the specified type.
      *
      * @param columnName the column name
      * @param type the class of the returned value
+     * @return the value
      */
     @Override
     public <T> T getObject(String columnName, Class<T> type) throws SQLException {
-        throw getUnsupportedException();
+        return getObject(findColumn(columnName), type);
     }
 
     /**
