@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -15,6 +15,7 @@ import java.io.LineNumberReader;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.io.Writer;
+import java.nio.charset.StandardCharsets;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.Map.Entry;
@@ -33,7 +34,7 @@ public class SortedProperties extends Properties {
 
     @Override
     public synchronized Enumeration<Object> keys() {
-        Vector<String> v = new Vector<String>();
+        Vector<String> v = new Vector<>();
         for (Object o : keySet()) {
             v.add(o.toString());
         }
@@ -51,10 +52,9 @@ public class SortedProperties extends Properties {
      */
     public static boolean getBooleanProperty(Properties prop, String key,
             boolean def) {
-        String value = prop.getProperty(key, "" + def);
         try {
-            return Boolean.parseBoolean(value);
-        } catch (Exception e) {
+            return Utils.parseBoolean(prop.getProperty(key, null), def, true);
+        } catch (IllegalArgumentException e) {
             e.printStackTrace();
             return def;
         }
@@ -104,7 +104,7 @@ public class SortedProperties extends Properties {
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         store(out, null);
         ByteArrayInputStream in = new ByteArrayInputStream(out.toByteArray());
-        InputStreamReader reader = new InputStreamReader(in, "ISO8859-1");
+        InputStreamReader reader = new InputStreamReader(in, StandardCharsets.ISO_8859_1);
         LineNumberReader r = new LineNumberReader(reader);
         Writer w;
         try {
@@ -112,17 +112,17 @@ public class SortedProperties extends Properties {
         } catch (Exception e) {
             throw new IOException(e.toString(), e);
         }
-        PrintWriter writer = new PrintWriter(new BufferedWriter(w));
-        while (true) {
-            String line = r.readLine();
-            if (line == null) {
-                break;
-            }
-            if (!line.startsWith("#")) {
-                writer.print(line + "\n");
+        try (PrintWriter writer = new PrintWriter(new BufferedWriter(w))) {
+            while (true) {
+                String line = r.readLine();
+                if (line == null) {
+                    break;
+                }
+                if (!line.startsWith("#")) {
+                    writer.print(line + "\n");
+                }
             }
         }
-        writer.close();
     }
 
     /**
@@ -132,7 +132,7 @@ public class SortedProperties extends Properties {
      */
     public synchronized String toLines() {
         StringBuilder buff = new StringBuilder();
-        for (Entry<Object, Object> e : new TreeMap<Object, Object>(this).entrySet()) {
+        for (Entry<Object, Object> e : new TreeMap<>(this).entrySet()) {
             buff.append(e.getKey()).append('=').append(e.getValue()).append('\n');
         }
         return buff.toString();
