@@ -1,25 +1,24 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: Daniel Gredler
  */
 package org.h2.util;
 
 import java.math.BigDecimal;
-import java.sql.Date;
-import java.sql.Timestamp;
+import java.text.DateFormatSymbols;
 import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
 import java.text.SimpleDateFormat;
 import java.util.Arrays;
-import java.util.Calendar;
 import java.util.Currency;
-import java.util.GregorianCalendar;
 import java.util.Locale;
 import java.util.TimeZone;
 
 import org.h2.api.ErrorCode;
 import org.h2.message.DbException;
+import org.h2.value.Value;
+import org.h2.value.ValueTimestampTimeZone;
 
 /**
  * Emulates Oracle's TO_CHAR function.
@@ -29,16 +28,40 @@ public class ToChar {
     /**
      * The beginning of the Julian calendar.
      */
-    private static final long JULIAN_EPOCH;
+    static final int JULIAN_EPOCH = -2_440_588;
 
-    static {
-        GregorianCalendar epoch = new GregorianCalendar(Locale.ENGLISH);
-        epoch.setGregorianChange(new Date(Long.MAX_VALUE));
-        epoch.clear();
-        epoch.set(4713, Calendar.JANUARY, 1, 0, 0, 0);
-        epoch.set(Calendar.ERA, GregorianCalendar.BC);
-        JULIAN_EPOCH = epoch.getTimeInMillis();
-    }
+    private static final int[] ROMAN_VALUES = { 1000, 900, 500, 400, 100, 90, 50, 40, 10, 9,
+            5, 4, 1 };
+
+    private static final String[] ROMAN_NUMERALS = { "M", "CM", "D", "CD", "C", "XC",
+            "L", "XL", "X", "IX", "V", "IV", "I" };
+
+    /**
+     * The month field.
+     */
+    static final int MONTHS = 0;
+
+    /**
+     * The month field (short form).
+     */
+    static final int SHORT_MONTHS = 1;
+
+    /**
+     * The weekday field.
+     */
+    static final int WEEKDAYS = 2;
+
+    /**
+     * The weekday field (short form).
+     */
+    static final int SHORT_WEEKDAYS = 3;
+
+    /**
+     * The AM / PM field.
+     */
+    static final int AM_PM = 4;
+
+    private static volatile String[][] NAMES;
 
     private ToChar() {
         // utility class
@@ -123,10 +146,10 @@ public class ToChar {
      * @return the formatted number
      */
     public static String toChar(BigDecimal number, String format,
-            String nlsParam) {
+            @SuppressWarnings("unused") String nlsParam) {
 
         // short-circuit logic for formats that don't follow common logic below
-        String formatUp = format != null ? format.toUpperCase() : null;
+        String formatUp = format != null ? StringUtils.toUpperEnglish(format) : null;
         if (formatUp == null || formatUp.equals("TM") || formatUp.equals("TM9")) {
             String s = number.toPlainString();
             return s.startsWith("0.") ? s.substring(1) : s;
@@ -172,7 +195,7 @@ public class ToChar {
             format = format.substring(0, format.length() - 2);
         }
 
-        int v = formatUp.indexOf("V");
+        int v = formatUp.indexOf('V');
         if (v >= 0) {
             int digits = 0;
             for (int i = v + 1; i < format.length(); i++) {
@@ -332,7 +355,7 @@ public class ToChar {
     }
 
     private static String zeroesAfterDecimalSeparator(BigDecimal number) {
-        final String numberStr = number.toString();
+        final String numberStr = number.toPlainString();
         final int idx = numberStr.indexOf('.');
         if (idx < 0) {
             return "";
@@ -410,14 +433,10 @@ public class ToChar {
     }
 
     private static String toRomanNumeral(int number) {
-        int[] values = new int[] { 1000, 900, 500, 400, 100, 90, 50, 40, 10, 9,
-                5, 4, 1 };
-        String[] numerals = new String[] { "M", "CM", "D", "CD", "C", "XC",
-                "L", "XL", "X", "IX", "V", "IV", "I" };
         StringBuilder result = new StringBuilder();
-        for (int i = 0; i < values.length; i++) {
-            int value = values[i];
-            String numeral = numerals[i];
+        for (int i = 0; i < ROMAN_VALUES.length; i++) {
+            int value = ROMAN_VALUES[i];
+            String numeral = ROMAN_NUMERALS[i];
             while (number >= value) {
                 result.append(numeral);
                 number -= value;
@@ -428,7 +447,7 @@ public class ToChar {
 
     private static String toHex(BigDecimal number, String format) {
 
-        boolean fillMode = !format.toUpperCase().startsWith("FM");
+        boolean fillMode = !StringUtils.toUpperEnglish(format).startsWith("FM");
         boolean uppercase = !format.contains("x");
         boolean zeroPadded = format.startsWith("0");
         int digits = 0;
@@ -445,7 +464,7 @@ public class ToChar {
             hex = StringUtils.pad("", digits + 1, "#", true);
         } else {
             if (uppercase) {
-                hex = hex.toUpperCase();
+                hex = StringUtils.toUpperEnglish(hex);
             }
             if (zeroPadded) {
                 hex = StringUtils.pad(hex, digits, "0", false);
@@ -458,6 +477,57 @@ public class ToChar {
         return hex;
     }
 
+    /**
+     * Get the date (month / weekday / ...) names.
+     *
+     * @param names the field
+     * @return the names
+     */
+    static String[] getDateNames(int names) {
+        String[][] result = NAMES;
+        if (result == null) {
+            result = new String[5][];
+            DateFormatSymbols dfs = DateFormatSymbols.getInstance();
+            result[MONTHS] = dfs.getMonths();
+            String[] months = dfs.getShortMonths();
+            for (int i = 0; i < 12; i++) {
+                String month = months[i];
+                if (month.endsWith(".")) {
+                    months[i] = month.substring(0, month.length() - 1);
+                }
+            }
+            result[SHORT_MONTHS] = months;
+            result[WEEKDAYS] = dfs.getWeekdays();
+            result[SHORT_WEEKDAYS] = dfs.getShortWeekdays();
+            result[AM_PM] = dfs.getAmPmStrings();
+            NAMES = result;
+        }
+        return result[names];
+    }
+
+    /**
+     * Returns time zone display name or ID for the specified date-time value.
+     *
+     * @param value
+     *            value
+     * @param tzd
+     *            if {@code true} return TZD (time zone region with Daylight Saving
+     *            Time information included), if {@code false} return TZR (time zone
+     *            region)
+     * @return time zone display name or ID
+     */
+    private static String getTimeZone(Value value, boolean tzd) {
+        if (!(value instanceof ValueTimestampTimeZone)) {
+            TimeZone tz = TimeZone.getDefault();
+            if (tzd) {
+                boolean daylight = tz.inDaylightTime(value.getTimestamp());
+                return tz.getDisplayName(daylight, TimeZone.SHORT);
+            }
+            return tz.getID();
+        }
+        return DateTimeUtils.timeZoneNameFromOffsetMins(((ValueTimestampTimeZone) value).getTimeZoneOffsetMins());
+    }
+
     /**
      * Emulates Oracle's TO_CHAR(datetime) function.
      *
@@ -590,19 +660,31 @@ public class ToChar {
      * See also TO_CHAR(datetime) and datetime format models
      * in the Oracle documentation.
      *
-     * @param ts the timestamp to format
+     * @param value the date-time value to format
      * @param format the format pattern to use (if any)
      * @param nlsParam the NLS parameter (if any)
      * @return the formatted timestamp
      */
-    public static String toChar(Timestamp ts, String format, String nlsParam) {
-
+    public static String toCharDateTime(Value value, String format, @SuppressWarnings("unused") String nlsParam) {
+        long[] a = DateTimeUtils.dateAndTimeFromValue(value);
+        long dateValue = a[0];
+        long timeNanos = a[1];
+        int year = DateTimeUtils.yearFromDateValue(dateValue);
+        int monthOfYear = DateTimeUtils.monthFromDateValue(dateValue);
+        int dayOfMonth = DateTimeUtils.dayFromDateValue(dateValue);
+        int posYear = Math.abs(year);
+        long second = timeNanos / 1_000_000_000;
+        int nanos = (int) (timeNanos - second * 1_000_000_000);
+        int minute = (int) (second / 60);
+        second -= minute * 60;
+        int hour = minute / 60;
+        minute -= hour * 60;
+        int h12 = (hour + 11) % 12 + 1;
+        boolean isAM = hour < 12;
         if (format == null) {
             format = "DD-MON-YY HH.MI.SS.FF PM";
         }
 
-        GregorianCalendar cal = new GregorianCalendar(Locale.ENGLISH);
-        cal.setTimeInMillis(ts.getTime());
         StringBuilder output = new StringBuilder();
         boolean fillMode = true;
 
@@ -613,197 +695,213 @@ public class ToChar {
                 // AD / BC
 
             if ((cap = containsAt(format, i, "A.D.", "B.C.")) != null) {
-                String era = cal.get(Calendar.ERA) == GregorianCalendar.AD ? "A.D." : "B.C.";
+                String era = year > 0 ? "A.D." : "B.C.";
                 output.append(cap.apply(era));
                 i += 4;
             } else if ((cap = containsAt(format, i, "AD", "BC")) != null) {
-                String era = cal.get(Calendar.ERA) == GregorianCalendar.AD ? "AD" : "BC";
+                String era = year > 0 ? "AD" : "BC";
                 output.append(cap.apply(era));
                 i += 2;
 
                 // AM / PM
 
             } else if ((cap = containsAt(format, i, "A.M.", "P.M.")) != null) {
-                String am = cal.get(Calendar.AM_PM) == Calendar.AM ? "A.M." : "P.M.";
+                String am = isAM ? "A.M." : "P.M.";
                 output.append(cap.apply(am));
                 i += 4;
             } else if ((cap = containsAt(format, i, "AM", "PM")) != null) {
-                String am = cal.get(Calendar.AM_PM) == Calendar.AM ? "AM" : "PM";
+                String am = isAM ? "AM" : "PM";
                 output.append(cap.apply(am));
                 i += 2;
 
                 // Long/short date/time format
 
-            } else if ((cap = containsAt(format, i, "DL")) != null) {
-                output.append(new SimpleDateFormat("EEEE, MMMM d, yyyy").format(ts));
+            } else if (containsAt(format, i, "DL") != null) {
+                String day = getDateNames(WEEKDAYS)[DateTimeUtils.getSundayDayOfWeek(dateValue)];
+                String month = getDateNames(MONTHS)[monthOfYear - 1];
+                output.append(day).append(", ").append(month).append(' ').append(dayOfMonth).append(", ");
+                StringUtils.appendZeroPadded(output, 4, posYear);
                 i += 2;
-            } else if ((cap = containsAt(format, i, "DS")) != null) {
-                output.append(new SimpleDateFormat("MM/dd/yyyy").format(ts));
+            } else if (containsAt(format, i, "DS") != null) {
+                StringUtils.appendZeroPadded(output, 2, monthOfYear);
+                output.append('/');
+                StringUtils.appendZeroPadded(output, 2, dayOfMonth);
+                output.append('/');
+                StringUtils.appendZeroPadded(output, 4, posYear);
                 i += 2;
-            } else if ((cap = containsAt(format, i, "TS")) != null) {
-                output.append(new SimpleDateFormat("h:mm:ss aa").format(ts));
+            } else if (containsAt(format, i, "TS") != null) {
+                output.append(h12).append(':');
+                StringUtils.appendZeroPadded(output, 2, minute);
+                output.append(':');
+                StringUtils.appendZeroPadded(output, 2, second);
+                output.append(' ');
+                output.append(getDateNames(AM_PM)[isAM ? 0 : 1]);
                 i += 2;
 
                 // Day
 
-            } else if ((cap = containsAt(format, i, "DDD")) != null) {
-                output.append(cal.get(Calendar.DAY_OF_YEAR));
+            } else if (containsAt(format, i, "DDD") != null) {
+                output.append(DateTimeUtils.getDayOfYear(dateValue));
                 i += 3;
-            } else if ((cap = containsAt(format, i, "DD")) != null) {
-                output.append(String.format("%02d",
-                        cal.get(Calendar.DAY_OF_MONTH)));
+            } else if (containsAt(format, i, "DD") != null) {
+                StringUtils.appendZeroPadded(output, 2, dayOfMonth);
                 i += 2;
             } else if ((cap = containsAt(format, i, "DY")) != null) {
-                String day = new SimpleDateFormat("EEE").format(ts).toUpperCase();
+                String day = getDateNames(SHORT_WEEKDAYS)[DateTimeUtils.getSundayDayOfWeek(dateValue)];
                 output.append(cap.apply(day));
                 i += 2;
             } else if ((cap = containsAt(format, i, "DAY")) != null) {
-                String day = new SimpleDateFormat("EEEE").format(ts);
+                String day = getDateNames(WEEKDAYS)[DateTimeUtils.getSundayDayOfWeek(dateValue)];
                 if (fillMode) {
                     day = StringUtils.pad(day, "Wednesday".length(), " ", true);
                 }
                 output.append(cap.apply(day));
                 i += 3;
-            } else if ((cap = containsAt(format, i, "D")) != null) {
-                output.append(cal.get(Calendar.DAY_OF_WEEK));
+            } else if (containsAt(format, i, "D") != null) {
+                output.append(DateTimeUtils.getSundayDayOfWeek(dateValue));
                 i += 1;
-            } else if ((cap = containsAt(format, i, "J")) != null) {
-                long millis = ts.getTime() - JULIAN_EPOCH;
-                long days = (long) Math.floor(millis / (1000 * 60 * 60 * 24));
-                output.append(days);
+            } else if (containsAt(format, i, "J") != null) {
+                output.append(DateTimeUtils.absoluteDayFromDateValue(dateValue) - JULIAN_EPOCH);
                 i += 1;
 
                 // Hours
 
-            } else if ((cap = containsAt(format, i, "HH24")) != null) {
-                output.append(new DecimalFormat("00").format(cal.get(Calendar.HOUR_OF_DAY)));
+            } else if (containsAt(format, i, "HH24") != null) {
+                StringUtils.appendZeroPadded(output, 2, hour);
                 i += 4;
-            } else if ((cap = containsAt(format, i, "HH12")) != null) {
-                output.append(new DecimalFormat("00").format(cal.get(Calendar.HOUR)));
+            } else if (containsAt(format, i, "HH12") != null) {
+                StringUtils.appendZeroPadded(output, 2, h12);
                 i += 4;
-            } else if ((cap = containsAt(format, i, "HH")) != null) {
-                output.append(new DecimalFormat("00").format(cal.get(Calendar.HOUR)));
+            } else if (containsAt(format, i, "HH") != null) {
+                StringUtils.appendZeroPadded(output, 2, h12);
                 i += 2;
 
                 // Minutes
 
-            } else if ((cap = containsAt(format, i, "MI")) != null) {
-                output.append(new DecimalFormat("00").format(cal.get(Calendar.MINUTE)));
+            } else if (containsAt(format, i, "MI") != null) {
+                StringUtils.appendZeroPadded(output, 2, minute);
                 i += 2;
 
                 // Seconds
 
-            } else if ((cap = containsAt(format, i, "SSSSS")) != null) {
-                int seconds = cal.get(Calendar.HOUR_OF_DAY) * 60 * 60;
-                seconds += cal.get(Calendar.MINUTE) * 60;
-                seconds += cal.get(Calendar.SECOND);
+            } else if (containsAt(format, i, "SSSSS") != null) {
+                int seconds = (int) (timeNanos / 1_000_000_000);
                 output.append(seconds);
                 i += 5;
-            } else if ((cap = containsAt(format, i, "SS")) != null) {
-                output.append(new DecimalFormat("00").format(cal.get(Calendar.SECOND)));
+            } else if (containsAt(format, i, "SS") != null) {
+                StringUtils.appendZeroPadded(output, 2, second);
                 i += 2;
 
                 // Fractional seconds
 
-            } else if ((cap = containsAt(format, i, "FF1", "FF2",
-                    "FF3", "FF4", "FF5", "FF6", "FF7", "FF8", "FF9")) != null) {
-                int x = Integer.parseInt(format.substring(i + 2, i + 3));
-                int ff = (int) (cal.get(Calendar.MILLISECOND) * Math.pow(10, x - 3));
-                output.append(ff);
+            } else if (containsAt(format, i, "FF1", "FF2",
+                    "FF3", "FF4", "FF5", "FF6", "FF7", "FF8", "FF9") != null) {
+                int x = format.charAt(i + 2) - '0';
+                int ff = (int) (nanos * Math.pow(10, x - 9));
+                StringUtils.appendZeroPadded(output, x, ff);
                 i += 3;
-            } else if ((cap = containsAt(format, i, "FF")) != null) {
-                output.append(cal.get(Calendar.MILLISECOND) * 1000);
+            } else if (containsAt(format, i, "FF") != null) {
+                StringUtils.appendZeroPadded(output, 9, nanos);
                 i += 2;
 
                 // Time zone
 
-            } else if ((cap = containsAt(format, i, "TZR")) != null) {
-                TimeZone tz = TimeZone.getDefault();
-                output.append(tz.getID());
+            } else if (containsAt(format, i, "TZR") != null) {
+                output.append(getTimeZone(value, false));
                 i += 3;
-            } else if ((cap = containsAt(format, i, "TZD")) != null) {
-                TimeZone tz = TimeZone.getDefault();
-                boolean daylight = tz.inDaylightTime(new java.util.Date());
-                output.append(tz.getDisplayName(daylight, TimeZone.SHORT));
+            } else if (containsAt(format, i, "TZD") != null) {
+                output.append(getTimeZone(value, true));
                 i += 3;
 
                 // Week
 
-            } else if ((cap = containsAt(format, i, "IW", "WW")) != null) {
-                output.append(cal.get(Calendar.WEEK_OF_YEAR));
+            } else if (containsAt(format, i, "IW", "WW") != null) {
+                output.append(DateTimeUtils.getWeekOfYear(dateValue, 0, 1));
                 i += 2;
-            } else if ((cap = containsAt(format, i, "W")) != null) {
-                int w = (int) (1 + Math.floor(cal.get(Calendar.DAY_OF_MONTH) / 7));
+            } else if (containsAt(format, i, "W") != null) {
+                int w = 1 + dayOfMonth / 7;
                 output.append(w);
                 i += 1;
 
                 // Year
 
-            } else if ((cap = containsAt(format, i, "Y,YYY")) != null) {
-                output.append(new DecimalFormat("#,###").format(getYear(cal)));
+            } else if (containsAt(format, i, "Y,YYY") != null) {
+                output.append(new DecimalFormat("#,###").format(posYear));
                 i += 5;
-            } else if ((cap = containsAt(format, i, "SYYYY")) != null) {
-                if (cal.get(Calendar.ERA) == GregorianCalendar.BC) {
+            } else if (containsAt(format, i, "SYYYY") != null) {
+                // Should be <= 0, but Oracle prints negative years with off-by-one difference
+                if (year < 0) {
                     output.append('-');
                 }
-                output.append(new DecimalFormat("0000").format(getYear(cal)));
+                StringUtils.appendZeroPadded(output, 4, posYear);
                 i += 5;
-            } else if ((cap = containsAt(format, i, "YYYY", "IYYY", "RRRR")) != null) {
-                output.append(new DecimalFormat("0000").format(getYear(cal)));
+            } else if (containsAt(format, i, "YYYY", "RRRR") != null) {
+                StringUtils.appendZeroPadded(output, 4, posYear);
                 i += 4;
-            } else if ((cap = containsAt(format, i, "YYY", "IYY")) != null) {
-                output.append(new DecimalFormat("000").format(getYear(cal) % 1000));
+            } else if (containsAt(format, i, "IYYY") != null) {
+                StringUtils.appendZeroPadded(output, 4, Math.abs(DateTimeUtils.getIsoWeekYear(dateValue)));
+                i += 4;
+            } else if (containsAt(format, i, "YYY") != null) {
+                StringUtils.appendZeroPadded(output, 3, posYear % 1000);
                 i += 3;
-            } else if ((cap = containsAt(format, i, "YY", "IY", "RR")) != null) {
-                output.append(new DecimalFormat("00").format(getYear(cal) % 100));
+            } else if (containsAt(format, i, "IYY") != null) {
+                StringUtils.appendZeroPadded(output, 3, Math.abs(DateTimeUtils.getIsoWeekYear(dateValue)) % 1000);
+                i += 3;
+            } else if (containsAt(format, i, "YY", "RR") != null) {
+                StringUtils.appendZeroPadded(output, 2, posYear % 100);
                 i += 2;
-            } else if ((cap = containsAt(format, i, "I", "Y")) != null) {
-                output.append(getYear(cal) % 10);
+            } else if (containsAt(format, i, "IY") != null) {
+                StringUtils.appendZeroPadded(output, 2, Math.abs(DateTimeUtils.getIsoWeekYear(dateValue)) % 100);
+                i += 2;
+            } else if (containsAt(format, i, "Y") != null) {
+                output.append(posYear % 10);
+                i += 1;
+            } else if (containsAt(format, i, "I") != null) {
+                output.append(Math.abs(DateTimeUtils.getIsoWeekYear(dateValue)) % 10);
                 i += 1;
 
                 // Month / quarter
 
             } else if ((cap = containsAt(format, i, "MONTH")) != null) {
-                String month = new SimpleDateFormat("MMMM").format(ts);
+                String month = getDateNames(MONTHS)[monthOfYear - 1];
                 if (fillMode) {
                     month = StringUtils.pad(month, "September".length(), " ", true);
                 }
                 output.append(cap.apply(month));
                 i += 5;
             } else if ((cap = containsAt(format, i, "MON")) != null) {
-                String month = new SimpleDateFormat("MMM").format(ts);
+                String month = getDateNames(SHORT_MONTHS)[monthOfYear - 1];
                 output.append(cap.apply(month));
                 i += 3;
-            } else if ((cap = containsAt(format, i, "MM")) != null) {
-                output.append(String.format("%02d", cal.get(Calendar.MONTH) + 1));
+            } else if (containsAt(format, i, "MM") != null) {
+                StringUtils.appendZeroPadded(output, 2, monthOfYear);
                 i += 2;
             } else if ((cap = containsAt(format, i, "RM")) != null) {
-                int month = cal.get(Calendar.MONTH) + 1;
-                output.append(cap.apply(toRomanNumeral(month)));
+                output.append(cap.apply(toRomanNumeral(monthOfYear)));
                 i += 2;
-            } else if ((cap = containsAt(format, i, "Q")) != null) {
-                int q = (int) (1 + Math.floor(cal.get(Calendar.MONTH) / 3));
+            } else if (containsAt(format, i, "Q") != null) {
+                int q = 1 + ((monthOfYear - 1) / 3);
                 output.append(q);
                 i += 1;
 
                 // Local radix character
 
-            } else if ((cap = containsAt(format, i, "X")) != null) {
+            } else if (containsAt(format, i, "X") != null) {
                 char c = DecimalFormatSymbols.getInstance().getDecimalSeparator();
                 output.append(c);
                 i += 1;
 
                 // Format modifiers
 
-            } else if ((cap = containsAt(format, i, "FM")) != null) {
+            } else if (containsAt(format, i, "FM") != null) {
                 fillMode = !fillMode;
                 i += 2;
-            } else if ((cap = containsAt(format, i, "FX")) != null) {
+            } else if (containsAt(format, i, "FX") != null) {
                 i += 2;
 
                 // Literal text
 
-            } else if ((cap = containsAt(format, i, "\"")) != null) {
+            } else if (containsAt(format, i, "\"") != null) {
                 for (i = i + 1; i < format.length(); i++) {
                     char c = format.charAt(i);
                     if (c != '"') {
@@ -833,14 +931,6 @@ public class ToChar {
         return output.toString();
     }
 
-    private static int getYear(Calendar cal) {
-        int year = cal.get(Calendar.YEAR);
-        if (cal.get(Calendar.ERA) == GregorianCalendar.BC) {
-            year--;
-        }
-        return year;
-    }
-
     /**
      * Returns a capitalization strategy if the specified string contains any of
      * the specified substrings at the specified index. The capitalization
@@ -885,7 +975,7 @@ public class ToChar {
     }
 
     /** Represents a capitalization / casing strategy. */
-    private enum Capitalization {
+    public enum Capitalization {
 
         /**
          * All letters are uppercased.
@@ -912,7 +1002,7 @@ public class ToChar {
          * @return the capitalization / casing strategy which should be used
          *         when the first and second letters have the specified casing
          */
-        public static Capitalization toCapitalization(Boolean up1, Boolean up2) {
+        static Capitalization toCapitalization(Boolean up1, Boolean up2) {
             if (up1 == null) {
                 return Capitalization.CAPITALIZE;
             } else if (up2 == null) {
@@ -936,12 +1026,12 @@ public class ToChar {
             }
             switch (this) {
             case UPPERCASE:
-                return s.toUpperCase();
+                return StringUtils.toUpperEnglish(s);
             case LOWERCASE:
-                return s.toLowerCase();
+                return StringUtils.toLowerEnglish(s);
             case CAPITALIZE:
                 return Character.toUpperCase(s.charAt(0)) +
-                        (s.length() > 1 ? s.toLowerCase().substring(1) : "");
+                        (s.length() > 1 ? StringUtils.toLowerEnglish(s).substring(1) : "");
             default:
                 throw new IllegalArgumentException(
                         "Unknown capitalization strategy: " + this);
