@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -11,19 +11,23 @@ import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.Reader;
+import java.nio.charset.StandardCharsets;
 import java.sql.PreparedStatement;
 import java.sql.SQLException;
+
 import org.h2.engine.Constants;
+import org.h2.engine.Mode;
 import org.h2.engine.SysProperties;
 import org.h2.message.DbException;
-import org.h2.mvstore.DataUtils;
 import org.h2.store.DataHandler;
 import org.h2.store.FileStore;
 import org.h2.store.FileStoreInputStream;
 import org.h2.store.FileStoreOutputStream;
 import org.h2.store.LobStorageFrontend;
 import org.h2.store.LobStorageInterface;
+import org.h2.store.RangeReader;
 import org.h2.store.fs.FileUtils;
+import org.h2.util.Bits;
 import org.h2.util.IOUtils;
 import org.h2.util.MathUtils;
 import org.h2.util.StringUtils;
@@ -52,7 +56,7 @@ public class ValueLobDb extends Value implements Value.ValueClob,
 
     private final String fileName;
     private final FileStore tempFile;
-    private int tableId;
+    private final int tableId;
     private int hash;
 
     //Arbonaut: 13.07.2016
@@ -82,6 +86,7 @@ public class ValueLobDb extends Value implements Value.ValueClob,
         this.handler = null;
         this.fileName = null;
         this.tempFile = null;
+        this.tableId = 0;
     }
 
     /**
@@ -107,12 +112,13 @@ public class ValueLobDb extends Value implements Value.ValueClob,
                 if (len == 0) {
                     break;
                 }
-                byte[] data = new String(buff, 0, len).getBytes("UTF-8");
+                byte[] data = new String(buff, 0, len).getBytes(StandardCharsets.UTF_8);
                 out.write(data);
                 tmpPrecision += len;
             }
         }
         this.precision = tmpPrecision;
+        this.tableId = 0;
     }
 
     /**
@@ -128,10 +134,9 @@ public class ValueLobDb extends Value implements Value.ValueClob,
         this.fileName = createTempLobFileName(handler);
         this.tempFile = this.handler.openFile(fileName, "rw", false);
         this.tempFile.autoDelete();
-        FileStoreOutputStream out = new FileStoreOutputStream(tempFile, null, null);
         long tmpPrecision = 0;
         boolean compress = this.handler.getLobCompressionAlgorithm(Value.BLOB) != null;
-        try {
+        try (FileStoreOutputStream out = new FileStoreOutputStream(tempFile, null, null)) {
             while (true) {
                 tmpPrecision += len;
                 out.write(buff, 0, len);
@@ -145,10 +150,9 @@ public class ValueLobDb extends Value implements Value.ValueClob,
                     break;
                 }
             }
-        } finally {
-            out.close();
         }
         this.precision = tmpPrecision;
+        this.tableId = 0;
     }
 
     private static String createTempLobFileName(DataHandler handler)
@@ -181,30 +185,33 @@ public class ValueLobDb extends Value implements Value.ValueClob,
      * except when converting to BLOB or CLOB.
      *
      * @param t the new type
+     * @param precision the precision
+     * @param mode the mode
+     * @param column the column (if any), used for to improve the error message if conversion fails
+     * @param enumerators the ENUM datatype enumerators (if any),
+     *        for dealing with ENUM conversions
      * @return the converted value
      */
     @Override
-    public Value convertTo(int t) {
+    public Value convertTo(int t, int precision, Mode mode, Object column, String[] enumerators) {
         if (t == type) {
             return this;
         } else if (t == Value.CLOB) {
             if (handler != null) {
-                Value copy = handler.getLobStorage().
+                return handler.getLobStorage().
                         createClob(getReader(), -1);
-                return copy;
             } else if (small != null) {
                 return ValueLobDb.createSmallLob(t, small);
             }
         } else if (t == Value.BLOB) {
             if (handler != null) {
-                Value copy = handler.getLobStorage().
+                return handler.getLobStorage().
                         createBlob(getInputStream(), -1);
-                return copy;
             } else if (small != null) {
                 return ValueLobDb.createSmallLob(t, small);
             }
         }
-        return super.convertTo(t);
+        return super.convertTo(t, precision, mode, column, null);
     }
 
     @Override
@@ -237,8 +244,7 @@ public class ValueLobDb extends Value implements Value.ValueClob,
     @Override
     public Value copy(DataHandler database, int tableId) {
         if (small == null) {
-            Value v2 = handler.getLobStorage().copyLob(this, tableId, getPrecision());
-            return v2;
+            return handler.getLobStorage().copyLob(this, tableId, getPrecision());
         } else if (small.length > database.getMaxLengthInplaceLob()) {
             LobStorageInterface s = database.getLobStorage();
             Value v;
@@ -281,7 +287,7 @@ public class ValueLobDb extends Value implements Value.ValueClob,
         try {
             if (type == Value.CLOB) {
                 if (small != null) {
-                    return new String(small, Constants.UTF8);
+                    return new String(small, StandardCharsets.UTF_8);
                 }
                 return IOUtils.readStringAndClose(getReader(), len);
             }
@@ -355,7 +361,7 @@ public class ValueLobDb extends Value implements Value.ValueClob,
             return Integer.signum(getString().compareTo(v.getString()));
         }
         byte[] v2 = v.getBytesNoCopy();
-        return Utils.compareNotNullSigned(getBytes(), v2);
+        return Bits.compareNotNullSigned(getBytesNoCopy(), v2);
     }
 
     @Override
@@ -371,6 +377,11 @@ public class ValueLobDb extends Value implements Value.ValueClob,
         return IOUtils.getBufferedReader(getInputStream());
     }
 
+    @Override
+    public Reader getReader(long oneBasedOffset, long length) {
+        return ValueLob.rangeReader(getReader(), oneBasedOffset, length, type == Value.CLOB ? precision : -1);
+    }
+
     @Override
     public InputStream getInputStream() {
         if (small != null) {
@@ -389,6 +400,29 @@ public class ValueLobDb extends Value implements Value.ValueClob,
         }
     }
 
+    @Override
+    public InputStream getInputStream(long oneBasedOffset, long length) {
+        long byteCount;
+        InputStream inputStream;
+        if (small != null) {
+            return super.getInputStream(oneBasedOffset, length);
+        } else if (fileName != null) {
+            FileStore store = handler.openFile(fileName, "r", true);
+            boolean alwaysClose = SysProperties.lobCloseBetweenReads;
+            byteCount = store.length();
+            inputStream = new BufferedInputStream(new FileStoreInputStream(store,
+                    handler, false, alwaysClose), Constants.IO_BUFFER_SIZE);
+        } else {
+            byteCount = (type == Value.BLOB) ? precision : -1;
+            try {
+                inputStream = handler.getLobStorage().getInputStream(this, hmac, byteCount);
+            } catch (IOException e) {
+                throw DbException.convertIOException(e, toString());
+            }
+        }
+        return ValueLob.rangeInputStream(inputStream, oneBasedOffset, length, byteCount);
+    }
+
     @Override
     public void set(PreparedStatement prep, int parameterIndex)
             throws SQLException {
@@ -508,6 +542,15 @@ public class ValueLobDb extends Value implements Value.ValueClob,
      */
     public static ValueLobDb createTempClob(Reader in, long length,
             DataHandler handler) {
+        if (length >= 0) {
+            // Otherwise BufferedReader may try to read more data than needed and that
+            // blocks the network level
+            try {
+                in = new RangeReader(in, 0, length);
+            } catch (IOException e) {
+                throw DbException.convert(e);
+            }
+        }
         BufferedReader reader;
         if (in instanceof BufferedReader) {
             reader = (BufferedReader) in;
@@ -532,12 +575,11 @@ public class ValueLobDb extends Value implements Value.ValueClob,
                 len = IOUtils.readFully(reader, buff, len);
             }
             if (len <= handler.getMaxLengthInplaceLob()) {
-                byte[] small = new String(buff, 0, len).getBytes(Constants.UTF8);
+                byte[] small = new String(buff, 0, len).getBytes(StandardCharsets.UTF_8);
                 return ValueLobDb.createSmallLob(Value.CLOB, small, len);
             }
             reader.reset();
-            ValueLobDb lob = new ValueLobDb(handler, reader, remaining);
-            return lob;
+            return new ValueLobDb(handler, reader, remaining);
         } catch (IOException e) {
             throw DbException.convertIOException(e, null);
         }
@@ -565,16 +607,14 @@ public class ValueLobDb extends Value implements Value.ValueClob,
                 buff = IOUtils.readBytesAndClose(in, -1);
                 len = buff.length;
             } else {
-                buff = DataUtils.newBytes(len);
+                buff = Utils.newBytes(len);
                 len = IOUtils.readFully(in, buff, len);
             }
             if (len <= handler.getMaxLengthInplaceLob()) {
-                byte[] small = DataUtils.newBytes(len);
-                System.arraycopy(buff, 0, small, 0, len);
+                byte[] small = Utils.copyBytes(buff, len);
                 return ValueLobDb.createSmallLob(Value.BLOB, small, small.length);
             }
-            ValueLobDb lob = new ValueLobDb(handler, buff, len, in, remaining);
-            return lob;
+            return new ValueLobDb(handler, buff, len, in, remaining);
         } catch (IOException e) {
             throw DbException.convertIOException(e, null);
         }
@@ -615,7 +655,7 @@ public class ValueLobDb extends Value implements Value.ValueClob,
                 try {
                     int p = MathUtils.convertLongToInt(precision);
                     String s = IOUtils.readStringAndClose(getReader(), p);
-                    byte[] data = s.getBytes(Constants.UTF8);
+                    byte[] data = s.getBytes(StandardCharsets.UTF_8);
                     lob = ValueLobDb.createSmallLob(type, data, s.length());
                 } catch (IOException e) {
                     throw DbException.convertIOException(e, null);
@@ -649,7 +689,7 @@ public class ValueLobDb extends Value implements Value.ValueClob,
     public static Value createSmallLob(int type, byte[] small) {
         int precision;
         if (type == Value.CLOB) {
-            precision = new String(small, Constants.UTF8).length();
+            precision = new String(small, StandardCharsets.UTF_8).length();
         } else {
             precision = small.length;
         }
