@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -11,7 +11,6 @@ import java.util.ArrayList;
 import java.util.Properties;
 import java.util.TimerTask;
 import java.util.concurrent.TimeUnit;
-
 import org.h2.Driver;
 import org.h2.engine.Constants;
 import org.h2.store.fs.FilePathRec;
@@ -46,6 +45,7 @@ import org.h2.test.db.TestLinkedTable;
 import org.h2.test.db.TestListener;
 import org.h2.test.db.TestLob;
 import org.h2.test.db.TestMemoryUsage;
+import org.h2.test.db.TestMergeUsing;
 import org.h2.test.db.TestMultiConn;
 import org.h2.test.db.TestMultiDimension;
 import org.h2.test.db.TestMultiThread;
@@ -54,6 +54,7 @@ import org.h2.test.db.TestOpenClose;
 import org.h2.test.db.TestOptimizations;
 import org.h2.test.db.TestOptimizerHints;
 import org.h2.test.db.TestOutOfMemory;
+import org.h2.test.db.TestPersistentCommonTableExpressions;
 import org.h2.test.db.TestPowerOff;
 import org.h2.test.db.TestQueryCache;
 import org.h2.test.db.TestReadOnly;
@@ -63,15 +64,15 @@ import org.h2.test.db.TestRights;
 import org.h2.test.db.TestRowFactory;
 import org.h2.test.db.TestRunscript;
 import org.h2.test.db.TestSQLInjection;
-import org.h2.test.db.TestScript;
-import org.h2.test.db.TestScriptSimple;
 import org.h2.test.db.TestSelectCountNonNullColumn;
 import org.h2.test.db.TestSequence;
 import org.h2.test.db.TestSessionsLocks;
+import org.h2.test.db.TestSetCollation;
 import org.h2.test.db.TestShow;
 import org.h2.test.db.TestSpaceReuse;
 import org.h2.test.db.TestSpatial;
 import org.h2.test.db.TestSpeed;
+import org.h2.test.db.TestSynonymForTable;
 import org.h2.test.db.TestTableEngines;
 import org.h2.test.db.TestTempTables;
 import org.h2.test.db.TestTransaction;
@@ -96,6 +97,7 @@ import org.h2.test.jdbc.TestConnection;
 import org.h2.test.jdbc.TestCustomDataTypesHandler;
 import org.h2.test.jdbc.TestDatabaseEventListener;
 import org.h2.test.jdbc.TestDriver;
+import org.h2.test.jdbc.TestGetGeneratedKeys;
 import org.h2.test.jdbc.TestJavaObject;
 import org.h2.test.jdbc.TestJavaObjectSerializer;
 import org.h2.test.jdbc.TestLimitUpdates;
@@ -123,6 +125,8 @@ import org.h2.test.mvcc.TestMvccMultiThreaded2;
 import org.h2.test.poweroff.TestReorderWrites;
 import org.h2.test.recover.RecoverLobTest;
 import org.h2.test.rowlock.TestRowLocks;
+import org.h2.test.scripts.TestScript;
+import org.h2.test.scripts.TestScriptSimple;
 import org.h2.test.server.TestAutoServer;
 import org.h2.test.server.TestInit;
 import org.h2.test.server.TestNestedLoop;
@@ -138,6 +142,7 @@ import org.h2.test.store.TestKillProcessWhileWriting;
 import org.h2.test.store.TestMVRTree;
 import org.h2.test.store.TestMVStore;
 import org.h2.test.store.TestMVStoreBenchmark;
+import org.h2.test.store.TestMVStoreStopCompact;
 import org.h2.test.store.TestMVStoreTool;
 import org.h2.test.store.TestMVTableEngine;
 import org.h2.test.store.TestObjectDataType;
@@ -172,6 +177,7 @@ import org.h2.test.unit.TestBitField;
 import org.h2.test.unit.TestBitStream;
 import org.h2.test.unit.TestBnf;
 import org.h2.test.unit.TestCache;
+import org.h2.test.unit.TestCharsetCollator;
 import org.h2.test.unit.TestClearReferences;
 import org.h2.test.unit.TestCollation;
 import org.h2.test.unit.TestCompress;
@@ -224,6 +230,7 @@ import org.h2.test.unit.TestValueHashMap;
 import org.h2.test.unit.TestValueMemory;
 import org.h2.test.utils.OutputCatcher;
 import org.h2.test.utils.SelfDestructor;
+import org.h2.test.utils.TestColumnNamer;
 import org.h2.tools.DeleteDbFiles;
 import org.h2.tools.Server;
 import org.h2.util.AbbaLockingDetector;
@@ -298,11 +305,6 @@ java org.h2.test.TestAll timer
      */
     public boolean memory;
 
-    /**
-     * Whether the test is running with code coverage.
-     */
-    public boolean coverage;
-
     /**
      * If code coverage is enabled.
      */
@@ -361,7 +363,12 @@ java org.h2.test.TestAll timer
     /**
      * If only fast/CI/Jenkins/Travis tests should be run.
      */
-    public boolean fast;
+    public boolean travis;
+
+    /**
+     * the vmlens.com race condition tool
+     */
+    public boolean vmlens;
 
     /**
      * The lock timeout to use
@@ -419,6 +426,10 @@ java org.h2.test.TestAll timer
      */
     String cacheType;
 
+    /** If not null the database should be opened with the collation parameter */
+    public String collation;
+
+
     /**
      * The AB-BA locking detector.
      */
@@ -431,6 +442,7 @@ java org.h2.test.TestAll timer
 
     private Server server;
 
+
     /**
      * Run all tests.
      *
@@ -463,8 +475,6 @@ java org.h2.test.TestAll timer
 
         // System.setProperty("h2.modifyOnWrite", "true");
 
-        // System.setProperty("h2.storeLocalTime", "true");
-
         // speedup
         // System.setProperty("h2.syncMethod", "");
 
@@ -498,8 +508,11 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
 */
         TestAll test = new TestAll();
         if (args.length > 0) {
-            if ("fast".equals(args[0])) {
-                test.fast = true;
+            if ("travis".equals(args[0])) {
+                test.travis = true;
+                test.testAll();
+            } else if ("vmlens".equals(args[0])) {
+                test.vmlens = true;
                 test.testAll();
             } else if ("reopen".equals(args[0])) {
                 System.setProperty("h2.delayWrongPasswordMin", "0");
@@ -532,7 +545,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
                 test.testEverything();
             } else if ("codeCoverage".equals(args[0])) {
                 test.codeCoverage = true;
-                test.runTests();
+                test.runCoverage();
             } else if ("multiThread".equals(args[0])) {
                 new TestMulti().runTest(test);
             } else if ("halt".equals(args[0])) {
@@ -549,7 +562,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
 
     private void testAll() throws Exception {
         runTests();
-        if (!fast) {
+        if (!travis && !vmlens) {
             Profiler prof = new Profiler();
             prof.depth = 16;
             prof.interval = 1;
@@ -595,8 +608,6 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
             abbaLockingDetector = new AbbaLockingDetector().startCollecting();
         }
 
-        coverage = isCoverage();
-
         smallLog = big = networked = memory = ssl = false;
         diskResult = traceSystemOut = diskUndo = false;
         traceTest = stopOnError = false;
@@ -609,6 +620,9 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         memory = true;
         multiThreaded = true;
         test();
+        if (vmlens) {
+            return;
+        }
         testUnit();
 
         // lazy
@@ -654,7 +668,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         defrag = true;
         test();
 
-        if (!fast) {
+        if (!travis) {
             traceLevelFile = 0;
             smallLog = true;
             networked = true;
@@ -676,21 +690,26 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
             cipher = null;
             test();
         }
-
     }
 
-    /**
-     * Check whether this method is running with "Emma" code coverage turned on.
-     *
-     * @return true if the stack trace contains ".emma."
-     */
-    private static boolean isCoverage() {
-        for (StackTraceElement e : Thread.currentThread().getStackTrace()) {
-            if (e.toString().contains(".emma.")) {
-                return true;
-            }
-        }
-        return false;
+    private void runCoverage() throws SQLException {
+        smallLog = big = networked = memory = ssl = false;
+        diskResult = traceSystemOut = diskUndo = false;
+        traceTest = stopOnError = false;
+        defrag = false;
+        traceLevelFile = throttle = 0;
+        cipher = null;
+
+        memory = true;
+        multiThreaded = true;
+        test();
+        testUnit();
+
+        multiThreaded = false;
+        mvStore = false;
+        mvcc = false;
+        test();
+        // testUnit();
     }
 
     /**
@@ -717,6 +736,9 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestCompatibilityOracle());
         addTest(new TestCsv());
         addTest(new TestDeadlock());
+        if (vmlens) {
+            return;
+        }
         addTest(new TestDrop());
         addTest(new TestDuplicateKeyUpdate());
         addTest(new TestEncryptedDb());
@@ -731,6 +753,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestLinkedTable());
         addTest(new TestListener());
         addTest(new TestLob());
+        addTest(new TestMergeUsing());
         addTest(new TestMultiConn());
         addTest(new TestMultiDimension());
         addTest(new TestMultiThreadedKernel());
@@ -741,6 +764,10 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestReadOnly());
         addTest(new TestRecursiveQueries());
         addTest(new TestGeneralCommonTableQueries());
+        if (!memory) {
+            // requires persistent store for reconnection tests
+            addTest(new TestPersistentCommonTableExpressions());
+        }
         addTest(new TestRights());
         addTest(new TestRunscript());
         addTest(new TestSQLInjection());
@@ -761,6 +788,9 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestViewAlterTable());
         addTest(new TestViewDropView());
         addTest(new TestReplace());
+        addTest(new TestSynonymForTable());
+        addTest(new TestColumnNamer());
+
 
         // jaqu
         addTest(new AliasMapTest());
@@ -786,10 +816,12 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestPreparedStatement());
         addTest(new TestResultSet());
         addTest(new TestStatement());
+        addTest(new TestGetGeneratedKeys());
         addTest(new TestTransactionIsolation());
         addTest(new TestUpdatableResultSet());
         addTest(new TestZloty());
         addTest(new TestCustomDataTypesHandler());
+        addTest(new TestSetCollation());
 
         // jdbcx
         addTest(new TestConnectionPool());
@@ -858,11 +890,11 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestMVRTree());
         addTest(new TestMVStore());
         addTest(new TestMVStoreBenchmark());
+        addTest(new TestMVStoreStopCompact());
         addTest(new TestMVStoreTool());
         addTest(new TestMVTableEngine());
         addTest(new TestObjectDataType());
         addTest(new TestRandomMapOps());
-        addTest(new TestReorderWrites());
         addTest(new TestSpinLock());
         addTest(new TestStreamStore());
         addTest(new TestTransactionStore());
@@ -875,6 +907,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestBitStream());
         addTest(new TestBnf());
         addTest(new TestCache());
+        addTest(new TestCharsetCollator());
         addTest(new TestClearReferences());
         addTest(new TestCollation());
         addTest(new TestCompress());
@@ -1026,8 +1059,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         DeleteDbFiles.execute(TestBase.BASE_TEST_DIR, null, true);
         FileUtils.deleteRecursive("trace.db", false);
         if (networked) {
-            String[] args = ssl ? new String[] { "-tcpSSL", "-tcpPort", "9192" }
-                    : new String[] { "-tcpPort", "9192" };
+            String[] args = ssl ? new String[] { "-tcpSSL" } : new String[0];
             server = Server.createTcpServer(args);
             try {
                 server.start();
@@ -1049,6 +1081,10 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         FileUtils.deleteRecursive(TestBase.BASE_TEST_DIR, true);
     }
 
+    public int getPort() {
+        return server == null ? 9192 : server.getPort();
+    }
+
     /**
      * Print system information.
      */
@@ -1079,7 +1115,6 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
     @Override
     public String toString() {
         StringBuilder buff = new StringBuilder();
-        appendIf(buff, fast, "fast");
         appendIf(buff, lazy, "lazy");
         appendIf(buff, mvStore, "mvStore");
         appendIf(buff, big, "big");
@@ -1102,6 +1137,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         appendIf(buff, stopOnError, "stopOnError");
         appendIf(buff, defrag, "defrag");
         appendIf(buff, splitFileSystem, "split");
+        appendIf(buff, collation != null, collation);
         return buff.toString();
     }
 
