@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -10,6 +10,8 @@ import java.io.FileReader;
 import java.io.FileWriter;
 import java.io.IOException;
 import java.io.LineNumberReader;
+import java.util.concurrent.TimeUnit;
+
 import org.h2.util.IOUtils;
 
 /**
@@ -25,7 +27,7 @@ public class Profile extends Thread {
     private boolean stop;
     private int maxIndex;
     private int lastIndex;
-    private long lastTime;
+    private long lastTimeNs;
     private BufferedWriter trace;
 
     private Profile() {
@@ -36,7 +38,7 @@ public class Profile extends Thread {
             maxIndex = r.getLineNumber();
             count = new int[maxIndex];
             time = new int[maxIndex];
-            lastTime = System.currentTimeMillis();
+            lastTimeNs = System.nanoTime();
             Runtime.getRuntime().addShutdownHook(this);
         } catch (Exception e) {
             e.printStackTrace();
@@ -75,7 +77,7 @@ public class Profile extends Thread {
      */
     public static void startCollecting() {
         MAIN.stop = false;
-        MAIN.lastTime = System.currentTimeMillis();
+        MAIN.lastTimeNs = System.nanoTime();
     }
 
     /**
@@ -106,10 +108,10 @@ public class Profile extends Thread {
         if (stop) {
             return;
         }
-        long now = System.currentTimeMillis();
+        long now = System.nanoTime();
         if (TRACE) {
             if (trace != null) {
-                int duration = (int) (now - lastTime);
+                long duration = TimeUnit.NANOSECONDS.toMillis(now - lastTimeNs);
                 try {
                     trace.write(i + "\t" + duration + "\r\n");
                 } catch (Exception e) {
@@ -119,8 +121,8 @@ public class Profile extends Thread {
             }
         }
         count[i]++;
-        time[lastIndex] += (int) (now - lastTime);
-        lastTime = now;
+        time[lastIndex] += (int) TimeUnit.NANOSECONDS.toMillis(now - lastTimeNs);
+        lastTimeNs = now;
         lastIndex = i;
     }
 
