@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -13,6 +13,8 @@ import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.Properties;
 
+
+
 /**
  * Tests the client info
  */
@@ -35,6 +37,7 @@ public class TestConnection extends TestBase {
         testSetSupportedClientInfoProperties();
         testSetUnsupportedClientInfoProperties();
         testSetInternalProperty();
+        testSetInternalPropertyToInitialValue();
         testSetGetSchema();
     }
 
@@ -45,15 +48,32 @@ public class TestConnection extends TestBase {
 
         assertThrows(SQLClientInfoException.class, conn).setClientInfo("numServers", "SomeValue");
         assertThrows(SQLClientInfoException.class, conn).setClientInfo("server23", "SomeValue");
+        conn.close();
+    }
+
+    /**
+     * Test that no exception is thrown if the client info of a connection
+     * managed in a connection pool is reset to its initial values.
+     *
+     * This is needed when using h2 in websphere liberty.
+     */
+    private void testSetInternalPropertyToInitialValue() throws SQLException {
+        // Use MySQL-mode since this allows all property names
+        // (apart from h2 internal names).
+        Connection conn = getConnection("clientInfoMySQL;MODE=MySQL");
+        String numServersPropertyName = "numServers";
+        String numServers = conn.getClientInfo(numServersPropertyName);
+        conn.setClientInfo(numServersPropertyName, numServers);
+        assertEquals(conn.getClientInfo(numServersPropertyName), numServers);
+        conn.close();
     }
 
     private void testSetUnsupportedClientInfoProperties() throws SQLException {
         Connection conn = getConnection("clientInfo");
-
         Properties properties = new Properties();
         properties.put("ClientUser", "someUser");
-
         assertThrows(SQLClientInfoException.class, conn).setClientInfo(properties);
+        conn.close();
     }
 
     private void testSetSupportedClientInfoProperties() throws SQLException {
@@ -67,6 +87,7 @@ public class TestConnection extends TestBase {
         assertNull(conn.getClientInfo("ApplicationName"));
         // new property has been set
         assertEquals(conn.getClientInfo("ClientUser"), "someUser");
+        conn.close();
     }
 
     private void testSetSupportedClientInfo() throws SQLException {
@@ -74,17 +95,20 @@ public class TestConnection extends TestBase {
         conn.setClientInfo("ApplicationName", "Connection Test");
 
         assertEquals(conn.getClientInfo("ApplicationName"), "Connection Test");
+        conn.close();
     }
 
     private void testSetUnsupportedClientInfo() throws SQLException {
         Connection conn = getConnection("clientInfoDB2;MODE=DB2");
         assertThrows(SQLClientInfoException.class, conn).setClientInfo(
                 "UnsupportedName", "SomeValue");
+        conn.close();
     }
 
     private void testGetUnsupportedClientInfo() throws SQLException {
         Connection conn = getConnection("clientInfo");
         assertNull(conn.getClientInfo("UnknownProperty"));
+        conn.close();
     }
 
     private void testSetGetSchema() throws SQLException {
