@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -34,7 +34,7 @@ public class TestCacheConcurrentLIRS extends TestBase {
     private void testConcurrent() {
         CacheLongKeyLIRS.Config cc = new CacheLongKeyLIRS.Config();
         cc.maxMemory = 100;
-        final CacheLongKeyLIRS<Integer> test = new CacheLongKeyLIRS<Integer>(cc);
+        final CacheLongKeyLIRS<Integer> test = new CacheLongKeyLIRS<>(cc);
         int threadCount = 8;
         final CountDownLatch wait = new CountDownLatch(1);
         final AtomicBoolean stopped = new AtomicBoolean();
