@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -12,7 +12,6 @@ import java.util.Map.Entry;
 import java.util.Random;
 import org.h2.dev.cache.CacheLIRS;
 import org.h2.test.TestBase;
-import org.h2.util.New;
 
 /**
  * Tests the cache algorithm.
@@ -485,7 +484,7 @@ public class TestCacheLIRS extends TestBase {
         Random r = new Random(1);
         for (int j = 0; j < 100; j++) {
             CacheLIRS<Integer, Integer> test = createCache(size / 2);
-            HashMap<Integer, Integer> good = New.hashMap();
+            HashMap<Integer, Integer> good = new HashMap<>();
             for (int i = 0; i < 10000; i++) {
                 int key = r.nextInt(size);
                 int value = r.nextInt();
@@ -557,7 +556,7 @@ public class TestCacheLIRS extends TestBase {
         List<K> cold = cache.keys(true, false);
         List<K> nonResident = cache.keys(true, true);
         assertEquals(nonResident.size(), cache.sizeNonResident());
-        HashSet<K> hot = new HashSet<K>(stack);
+        HashSet<K> hot = new HashSet<>(stack);
         hot.removeAll(cold);
         hot.removeAll(nonResident);
         assertEquals(hot.size(), cache.sizeHot());
@@ -569,7 +568,7 @@ public class TestCacheLIRS extends TestBase {
     }
 
     private static <K, V> CacheLIRS<K, V> createCache(int maxSize) {
-        return new CacheLIRS<K, V>(maxSize, 1, 0);
+        return new CacheLIRS<>(maxSize, 1, 0);
     }
 
 }
