@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -8,12 +8,10 @@ package org.h2.test.store;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
 import java.util.concurrent.TimeUnit;
-
 import org.h2.mvstore.MVStore;
 import org.h2.test.TestBase;
 import org.h2.util.New;
@@ -40,7 +38,7 @@ public class TestMVStoreBenchmark extends TestBase {
         if (!config.big) {
             return;
         }
-        if (config.coverage || config.codeCoverage) {
+        if (config.codeCoverage) {
             // run only when _not_ using a code coverage tool,
             // because the tool might instrument our code but not
             // java.util.*
@@ -124,14 +122,6 @@ public class TestMVStoreBenchmark extends TestBase {
     }
 
     static long getMemory() {
-        try {
-            LinkedList<byte[]> list = new LinkedList<byte[]>();
-            while (true) {
-                list.add(new byte[1024]);
-            }
-        } catch (OutOfMemoryError e) {
-            // ok
-        }
         for (int i = 0; i < 16; i++) {
             System.gc();
             try {
@@ -156,10 +146,10 @@ public class TestMVStoreBenchmark extends TestBase {
             MVStore store = MVStore.open(null);
             map = store.openMap("test");
             mv = testPerformance(map, size);
-            map = new HashMap<Integer, String>(size);
+            map = new HashMap<>(size);
             // map = new ConcurrentHashMap<Integer, String>(size);
             hash = testPerformance(map, size);
-            map = new TreeMap<Integer, String>();
+            map = new TreeMap<>();
             // map = new ConcurrentSkipListMap<Integer, String>();
             tree = testPerformance(map, size);
             if (hash < tree && mv < tree * 1.5) {
