@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -803,7 +803,7 @@ public class TestPageStore extends TestBase {
                 "NAME VARCHAR DEFAULT 'Hello World');");
         stat.execute("CREATE TABLE TEST(ID INT PRIMARY KEY, " +
                 "NAME VARCHAR DEFAULT 'Hello World')");
-        Set<Integer> rows = new TreeSet<Integer>();
+        Set<Integer> rows = new TreeSet<>();
         Random random = new Random(seed);
         for (int i = 0; i < len; i++) {
             int op = random.nextInt(3);
