@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -18,8 +18,10 @@ import java.io.Writer;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.math.BigDecimal;
+import java.math.BigInteger;
 import java.net.ServerSocket;
 import java.net.Socket;
+import java.nio.charset.StandardCharsets;
 import java.sql.Blob;
 import java.sql.Clob;
 import java.sql.Connection;
@@ -35,6 +37,8 @@ import java.sql.Types;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Random;
+import java.util.UUID;
+
 import org.h2.api.ErrorCode;
 import org.h2.engine.SysProperties;
 import org.h2.store.FileLister;
@@ -56,6 +60,7 @@ import org.h2.tools.SimpleResultSet;
 import org.h2.tools.SimpleResultSet.SimpleArray;
 import org.h2.util.JdbcUtils;
 import org.h2.util.Task;
+import org.h2.value.ValueUuid;
 
 /**
  * Tests the database tools.
@@ -64,7 +69,7 @@ public class TestTools extends TestBase {
 
     private static String lastUrl;
     private Server server;
-    private List<Server> remainingServers = new ArrayList<Server>(3);
+    private List<Server> remainingServers = new ArrayList<>(3);
 
     /**
      * Run just this test.
@@ -134,7 +139,7 @@ public class TestTools extends TestBase {
             c.runTool("-web", "-webPort", "9002", "-tool", "-browser", "-tcp",
                     "-tcpPort", "9003", "-pg", "-pgPort", "9004");
             assertContains(lastUrl, ":9002");
-            c.shutdown();
+            shutdownConsole(c);
 
             // check if starting the browser works
             c.runTool("-web", "-webPort", "9002", "-tool");
@@ -164,7 +169,7 @@ public class TestTools extends TestBase {
                 // ignore
             }
 
-            c.shutdown();
+            shutdownConsole(c);
 
             // trying to use the same port for two services should fail,
             // but also stop the first service
@@ -179,7 +184,19 @@ public class TestTools extends TestBase {
             } else {
                 System.clearProperty(SysProperties.H2_BROWSER);
             }
-            c.shutdown();
+            shutdownConsole(c);
+        }
+    }
+
+    private static void shutdownConsole(Console c) {
+        c.shutdown();
+        if (Thread.currentThread().isInterrupted()) {
+            // Clear interrupted state so test can continue its work safely
+            try {
+                Thread.sleep(1);
+            } catch (InterruptedException e) {
+                // Ignore
+            }
         }
     }
 
@@ -253,6 +270,8 @@ public class TestTools extends TestBase {
         Clob clob = new SimpleClob("Hello World");
         Blob blob = new SimpleBlob(new byte[]{(byte) 1, (byte) 2});
         rs.addRow(1, b, true, d, "10.3", Math.PI, "-3", a, t, ts, clob, blob);
+        rs.addRow(BigInteger.ONE, null, true, null, BigDecimal.ONE, 1d, null, null, null, null, null);
+        rs.addRow(BigInteger.ZERO, null, false, null, BigDecimal.ZERO, 0d, null, null, null, null, null);
         rs.addRow(null, null, null, null, null, null, null, null, null, null, null);
 
         rs.next();
@@ -266,6 +285,7 @@ public class TestTools extends TestBase {
         assertEquals((short) 1, rs.getShort("a"));
         assertTrue(rs.getObject(1).getClass() == Integer.class);
         assertTrue(rs.getObject("a").getClass() == Integer.class);
+        assertTrue(rs.getBoolean(1));
 
         assertEquals(b, rs.getBytes(2));
         assertEquals(b, rs.getBytes("b"));
@@ -284,6 +304,7 @@ public class TestTools extends TestBase {
         assertTrue(Math.PI == rs.getDouble("f"));
         assertTrue((float) Math.PI == rs.getFloat(6));
         assertTrue((float) Math.PI == rs.getFloat("f"));
+        assertTrue(rs.getBoolean(6));
 
         assertEquals(-3, rs.getInt(7));
         assertEquals(-3, rs.getByte(7));
@@ -322,6 +343,20 @@ public class TestTools extends TestBase {
 
         rs.next();
 
+        assertTrue(rs.getBoolean(1));
+        assertTrue(rs.getBoolean(3));
+        assertTrue(rs.getBoolean(5));
+        assertTrue(rs.getBoolean(6));
+
+        rs.next();
+
+        assertFalse(rs.getBoolean(1));
+        assertFalse(rs.getBoolean(3));
+        assertFalse(rs.getBoolean(5));
+        assertFalse(rs.getBoolean(6));
+
+        rs.next();
+
         assertEquals(0, rs.getLong(1));
         assertTrue(rs.wasNull());
         assertEquals(null, rs.getBytes(2));
@@ -450,6 +485,8 @@ public class TestTools extends TestBase {
         assertFalse(rs.isClosed());
         assertEquals(1, rs.getRow());
         assertTrue(rs.next());
+        assertTrue(rs.next());
+        assertTrue(rs.next());
         assertFalse(rs.next());
         assertThrows(ErrorCode.NO_DATA_AVAILABLE, (ResultSet) rs).
                 getInt(1);
@@ -457,6 +494,13 @@ public class TestTools extends TestBase {
         assertFalse(rs.isClosed());
         rs.close();
         assertTrue(rs.isClosed());
+        rs = new SimpleResultSet();
+        rs.addColumn("TEST", Types.BINARY, 0, 0);
+        UUID uuid = UUID.randomUUID();
+        rs.addRow(uuid);
+        rs.next();
+        assertEquals(uuid, rs.getObject(1));
+        assertEquals(uuid, ValueUuid.get(rs.getBytes(1)).getObject());
     }
 
     private void testJdbcDriverUtils() {
@@ -525,7 +569,7 @@ public class TestTools extends TestBase {
 
     private void testServerMain() throws SQLException {
         testNonSSL();
-        if (!config.fast) {
+        if (!config.travis) {
             testSSL();
         }
     }
@@ -720,7 +764,7 @@ public class TestTools extends TestBase {
         assertEquals(new byte[] { (byte) 10, (byte) 20 }, rs.getBytes("f"));
         assertEquals("2007-12-31", rs.getString("g"));
         assertEquals("23:59:59", rs.getString("h"));
-        assertEquals("2007-12-31 23:59:59.0", rs.getString("i"));
+        assertEquals("2007-12-31 23:59:59", rs.getString("i"));
         assertFalse(rs.next());
         conn.close();
     }
@@ -815,10 +859,9 @@ public class TestTools extends TestBase {
         int count = getSize(2, 10);
         for (int i = 0; i < count; i++) {
             Server tcpServer = Server.
-                    createTcpServer("-tcpPort", "9192").start();
+                    createTcpServer().start();
             tcpServer.stop();
-            tcpServer = Server.createTcpServer("-tcpPassword", "abc",
-                    "-tcpPort", "9192").start();
+            tcpServer = Server.createTcpServer("-tcpPassword", "abc").start();
             tcpServer.stop();
         }
     }
@@ -850,7 +893,7 @@ public class TestTools extends TestBase {
         byte[] large = new byte[getSize(10 * 1024, 100 * 1024)];
         random.nextBytes(large);
         prep.setBytes(2, large);
-        String largeText = new String(large, "ISO-8859-1");
+        String largeText = new String(large, StandardCharsets.ISO_8859_1);
         prep.setString(3, largeText);
         prep.execute();
 
@@ -1055,67 +1098,67 @@ public class TestTools extends TestBase {
             deleteDb("test");
             Server tcpServer = Server.createTcpServer(
                             "-baseDir", getBaseDir(),
-                            "-tcpPort", "9192",
                             "-tcpAllowOthers").start();
             remainingServers.add(tcpServer);
-            conn = getConnection("jdbc:h2:tcp://localhost:9192/test", "sa", "");
+            final int port = tcpServer.getPort();
+            conn = getConnection("jdbc:h2:tcp://localhost:"+ port +"/test", "sa", "");
             conn.close();
             // must not be able to use a different base dir
             new AssertThrows(ErrorCode.IO_EXCEPTION_1) {
                 @Override
                 public void test() throws SQLException {
-                    getConnection("jdbc:h2:tcp://localhost:9192/../test", "sa", "");
+                    getConnection("jdbc:h2:tcp://localhost:"+ port +"/../test", "sa", "");
             }};
             new AssertThrows(ErrorCode.IO_EXCEPTION_1) {
                 @Override
                 public void test() throws SQLException {
-                    getConnection("jdbc:h2:tcp://localhost:9192/../test2/test", "sa", "");
+                    getConnection("jdbc:h2:tcp://localhost:"+port+"/../test2/test", "sa", "");
             }};
             tcpServer.stop();
             Server tcpServerWithPassword = Server.createTcpServer(
                             "-ifExists",
                             "-tcpPassword", "abc",
-                            "-baseDir", getBaseDir(),
-                            "-tcpPort", "9192").start();
+                            "-baseDir", getBaseDir()).start();
+            final int prt = tcpServerWithPassword.getPort();
             remainingServers.add(tcpServerWithPassword);
             // must not be able to create new db
             new AssertThrows(ErrorCode.DATABASE_NOT_FOUND_1) {
                 @Override
                 public void test() throws SQLException {
-                    getConnection("jdbc:h2:tcp://localhost:9192/test2", "sa", "");
+                    getConnection("jdbc:h2:tcp://localhost:"+prt+"/test2", "sa", "");
             }};
             new AssertThrows(ErrorCode.DATABASE_NOT_FOUND_1) {
                 @Override
                 public void test() throws SQLException {
-                    getConnection("jdbc:h2:tcp://localhost:9192/test2;ifexists=false", "sa", "");
+                    getConnection("jdbc:h2:tcp://localhost:"+prt+"/test2;ifexists=false", "sa", "");
             }};
-            conn = getConnection("jdbc:h2:tcp://localhost:9192/test", "sa", "");
+            conn = getConnection("jdbc:h2:tcp://localhost:"+prt+"/test", "sa", "");
             conn.close();
             new AssertThrows(ErrorCode.WRONG_USER_OR_PASSWORD) {
                 @Override
                 public void test() throws SQLException {
-                    Server.shutdownTcpServer("tcp://localhost:9192", "", true, false);
+                    Server.shutdownTcpServer("tcp://localhost:"+prt, "", true, false);
             }};
-            conn = getConnection("jdbc:h2:tcp://localhost:9192/test", "sa", "");
+            conn = getConnection("jdbc:h2:tcp://localhost:"+prt+"/test", "sa", "");
             // conn.close();
-            Server.shutdownTcpServer("tcp://localhost:9192", "abc", true, false);
+            Server.shutdownTcpServer("tcp://localhost:"+prt, "abc", true, false);
             // check that the database is closed
             deleteDb("test");
             // server must have been closed
             assertThrows(ErrorCode.CONNECTION_BROKEN_1, this).
-                    getConnection("jdbc:h2:tcp://localhost:9192/test", "sa", "");
+                    getConnection("jdbc:h2:tcp://localhost:"+prt+"/test", "sa", "");
             JdbcUtils.closeSilently(conn);
             // Test filesystem prefix and escape from baseDir
             deleteDb("testSplit");
             server = Server.createTcpServer(
                             "-baseDir", getBaseDir(),
-                            "-tcpPort", "9192",
                             "-tcpAllowOthers").start();
-            conn = getConnection("jdbc:h2:tcp://localhost:9192/split:testSplit", "sa", "");
+            final int p = server.getPort();
+            conn = getConnection("jdbc:h2:tcp://localhost:"+p+"/split:testSplit", "sa", "");
             conn.close();
 
             assertThrows(ErrorCode.IO_EXCEPTION_1, this).
-                    getConnection("jdbc:h2:tcp://localhost:9192/../test", "sa", "");
+                    getConnection("jdbc:h2:tcp://localhost:"+p+"/../test", "sa", "");
 
             server.stop();
             deleteDb("testSplit");
