@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -8,17 +8,23 @@ package org.h2.test.unit;
 import java.math.BigDecimal;
 import java.sql.Connection;
 import java.sql.Date;
+import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Time;
 import java.sql.Timestamp;
 import java.sql.Types;
+import java.util.Arrays;
+import java.util.Calendar;
+import java.util.TimeZone;
 import java.util.UUID;
+
 import org.h2.api.ErrorCode;
 import org.h2.message.DbException;
 import org.h2.test.TestBase;
 import org.h2.test.utils.AssertThrows;
 import org.h2.tools.SimpleResultSet;
+import org.h2.util.Bits;
 import org.h2.value.DataType;
 import org.h2.value.Value;
 import org.h2.value.ValueArray;
@@ -31,6 +37,7 @@ import org.h2.value.ValueLobDb;
 import org.h2.value.ValueNull;
 import org.h2.value.ValueResultSet;
 import org.h2.value.ValueString;
+import org.h2.value.ValueTimestamp;
 import org.h2.value.ValueUuid;
 
 /**
@@ -50,12 +57,14 @@ public class TestValue extends TestBase {
     @Override
     public void test() throws SQLException {
         testResultSetOperations();
+        testBinaryAndUuid();
         testCastTrim();
         testValueResultSet();
         testDataType();
         testUUID();
         testDouble(false);
         testDouble(true);
+        testTimestamp();
         testModulusDouble();
         testModulusDecimal();
         testModulusOperator();
@@ -87,6 +96,7 @@ public class TestValue extends TestBase {
         testResultSetOperation(new Time(7));
         testResultSetOperation(new Timestamp(8));
         testResultSetOperation(new BigDecimal("9"));
+        testResultSetOperation(UUID.randomUUID());
 
         SimpleResultSet rs2 = new SimpleResultSet();
         rs2.setAutoClose(false);
@@ -114,6 +124,28 @@ public class TestValue extends TestBase {
         }
     }
 
+    private void testBinaryAndUuid() throws SQLException {
+        try (Connection conn = getConnection("binaryAndUuid")) {
+            UUID uuid = UUID.randomUUID();
+            PreparedStatement prep;
+            ResultSet rs;
+            // Check conversion to byte[]
+            prep = conn.prepareStatement("SELECT * FROM TABLE(X BINARY=?)");
+            prep.setObject(1, new Object[] { uuid });
+            rs = prep.executeQuery();
+            rs.next();
+            assertTrue(Arrays.equals(Bits.uuidToBytes(uuid), (byte[]) rs.getObject(1)));
+            // Check that type is not changed
+            prep = conn.prepareStatement("SELECT * FROM TABLE(X UUID=?)");
+            prep.setObject(1, new Object[] { uuid });
+            rs = prep.executeQuery();
+            rs.next();
+            assertEquals(uuid, rs.getObject(1));
+        } finally {
+            deleteDb("binaryAndUuid");
+        }
+    }
+
     private void testCastTrim() {
         Value v;
         String spaces = new String(new char[100]).replace((char) 0, ' ');
@@ -265,7 +297,7 @@ public class TestValue extends TestBase {
             values[i] = v;
             assertTrue(values[i].compareTypeSafe(values[i], null) == 0);
             assertTrue(v.equals(v));
-            assertEquals(i < 2 ? -1 : i > 2 ? 1 : 0, v.getSignum());
+            assertEquals(Integer.compare(i, 2), v.getSignum());
         }
         for (int i = 0; i < d.length - 1; i++) {
             assertTrue(values[i].compareTypeSafe(values[i+1], null) < 0);
@@ -274,6 +306,30 @@ public class TestValue extends TestBase {
         }
     }
 
+    private void testTimestamp() {
+        ValueTimestamp valueTs = ValueTimestamp.parse("2000-01-15 10:20:30.333222111");
+        Timestamp ts = Timestamp.valueOf("2000-01-15 10:20:30.333222111");
+        assertEquals(ts.toString(), valueTs.getString());
+        assertEquals(ts, valueTs.getTimestamp());
+        Calendar c = Calendar.getInstance(TimeZone.getTimeZone("Europe/Berlin"));
+        c.set(2018, 02, 25, 1, 59, 00);
+        c.set(Calendar.MILLISECOND, 123);
+        long expected = c.getTimeInMillis();
+        ts = ValueTimestamp.parse("2018-03-25 01:59:00.123123123 Europe/Berlin").getTimestamp();
+        assertEquals(expected, ts.getTime());
+        assertEquals(123123123, ts.getNanos());
+        ts = ValueTimestamp.parse("2018-03-25 01:59:00.123123123+01").getTimestamp();
+        assertEquals(expected, ts.getTime());
+        assertEquals(123123123, ts.getNanos());
+        expected += 60000; // 1 minute
+        ts = ValueTimestamp.parse("2018-03-25 03:00:00.123123123 Europe/Berlin").getTimestamp();
+        assertEquals(expected, ts.getTime());
+        assertEquals(123123123, ts.getNanos());
+        ts = ValueTimestamp.parse("2018-03-25 03:00:00.123123123+02").getTimestamp();
+        assertEquals(expected, ts.getTime());
+        assertEquals(123123123, ts.getNanos());
+    }
+
     private void testUUID() {
         long maxHigh = 0, maxLow = 0, minHigh = -1L, minLow = -1L;
         for (int i = 0; i < 100; i++) {
@@ -326,13 +382,11 @@ public class TestValue extends TestBase {
     }
 
     private void testModulusOperator() throws SQLException {
-        Connection conn = getConnection("modulus");
-        try {
+        try (Connection conn = getConnection("modulus")) {
             ResultSet rs = conn.createStatement().executeQuery("CALL 12 % 10");
             rs.next();
             assertEquals(2, rs.getInt(1));
         } finally {
-            conn.close();
             deleteDb("modulus");
         }
     }
