@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -81,7 +81,7 @@ public class FtpServer extends Tool implements Service {
     private String writeUserName = DEFAULT_WRITE,
             writePassword = DEFAULT_WRITE_PASSWORD;
     private String readUserName = DEFAULT_READ;
-    private final HashMap<String, Process> tasks = new HashMap<String, Process>();
+    private final HashMap<String, Process> tasks = new HashMap<>();
 
     private boolean trace;
     private boolean allowTask;
