@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -7,7 +7,7 @@ package org.h2.dev.hash;
 
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
-import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
 import java.security.SecureRandom;
 import java.util.ArrayList;
 import java.util.Set;
@@ -330,8 +330,7 @@ public class MinimalPerfectHash<K> {
      * @return the hash function description
      */
     public static <K> byte[] generate(Set<K> set, UniversalHash<K> hash) {
-        ArrayList<K> list = new ArrayList<K>();
-        list.addAll(set);
+        ArrayList<K> list = new ArrayList<>(set);
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         int seed = RANDOM.nextInt();
         out.write(seed >>> 24);
@@ -409,7 +408,7 @@ public class MinimalPerfectHash<K> {
         boolean isRoot = level == 0;
         ArrayList<ArrayList<K>> lists;
         do {
-            lists = new ArrayList<ArrayList<K>>(split);
+            lists = new ArrayList<>(split);
             for (int i = 0; i < split; i++) {
                 lists.add(new ArrayList<K>(size / split));
             }
@@ -452,11 +451,11 @@ public class MinimalPerfectHash<K> {
             final int seed,
             ByteArrayOutputStream out) {
         final ArrayList<ByteArrayOutputStream> outList =
-                new ArrayList<ByteArrayOutputStream>();
+                new ArrayList<>();
         int processors = Runtime.getRuntime().availableProcessors();
         Thread[] threads = new Thread[processors];
         final AtomicInteger success = new AtomicInteger();
-        final AtomicReference<Exception> failure = new AtomicReference<Exception>();
+        final AtomicReference<Exception> failure = new AtomicReference<>();
         for (int i = 0; i < processors; i++) {
             threads[i] = new Thread() {
                 @Override
@@ -528,7 +527,7 @@ public class MinimalPerfectHash<K> {
         return (x & (-1 >>> 1)) % size;
     }
 
-    private static <K> int hash(int x, int level, int offset, int size) {
+    private static int hash(int x, int level, int offset, int size) {
         x += level + offset * 32;
         x = ((x >>> 16) ^ x) * 0x45d9f3b;
         x = ((x >>> 16) ^ x) * 0x45d9f3b;
@@ -677,8 +676,6 @@ public class MinimalPerfectHash<K> {
      */
     public static class StringHash implements UniversalHash<String> {
 
-        private static final Charset UTF8 = Charset.forName("UTF-8");
-
         @Override
         public int hashCode(String o, int index, int seed) {
             if (index == 0) {
@@ -723,7 +720,7 @@ public class MinimalPerfectHash<K> {
          * @return the hash value
          */
         public static int getSipHash24(String o, long k0, long k1) {
-            byte[] b = o.getBytes(UTF8);
+            byte[] b = o.getBytes(StandardCharsets.UTF_8);
             return getSipHash24(b, 0, b.length, k0, k1);
         }
 
