@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -71,7 +71,7 @@ public class ThreadDumpCleaner {
 
     };
 
-    private ArrayList<Pattern> patterns = new ArrayList<Pattern>();
+    private final ArrayList<Pattern> patterns = new ArrayList<>();
 
     {
         for (String s : PATTERN) {
