@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -23,7 +23,7 @@ public class QueryJoinCondition<A> {
     }
 
     public Query<?> is(A y) {
-        join.addConditionToken(new Condition<A>(x, y, CompareType.EQUAL));
+        join.addConditionToken(new Condition<>(x, y, CompareType.EQUAL));
         return query;
     }
 }
